@@ -50,28 +50,20 @@ class CalculationThread(QThread):
         @param gui: GUI
         """
         QThread.__init__(self)
-        Debug(self, ": Init")
+        Debug(self, ": Init", init=True)
         self.gui = gui
 
-        # Connect progress update signal and create callback:
+        # Connect progress update signal
+        self._progress_update.connect(  # type: ignore
+            lambda x: self.gui.statusbar.set_progress(x)
+        )
 
-        # noinspection PyUnresolvedReferences
-        self._progress_update.connect(lambda x: self.gui.statusbar.set_progress(x))
-        # noinspection PyUnresolvedReferences
-        self.progress_callback = self._progress_update.emit
-
-        # Connect valid state signals to corresponding handlers:
-
-        # noinspection PyUnresolvedReferences
-        self._wire_valid.connect(self.on_wire_valid)
-        # noinspection PyUnresolvedReferences
-        self._sampling_volume_valid.connect(self.on_sampling_volume_valid)
-        # noinspection PyUnresolvedReferences
-        self._field_valid.connect(self.on_field_valid)
-        # noinspection PyUnresolvedReferences
-        self._metric_valid.connect(self.on_metric_valid)
-        # noinspection PyUnresolvedReferences
-        self._parameters_valid.connect(self.on_parameters_valid)
+        # Connect valid state signals
+        self._wire_valid.connect(self.on_wire_valid)  # type: ignore
+        self._sampling_volume_valid.connect(self.on_sampling_volume_valid)  # type: ignore
+        self._field_valid.connect(self.on_field_valid)  # type: ignore
+        self._metric_valid.connect(self.on_metric_valid)  # type: ignore
+        self._parameters_valid.connect(self.on_parameters_valid)  # type: ignore
 
     def run(self) -> None:
         """
@@ -81,78 +73,84 @@ class CalculationThread(QThread):
 
         with ModelAccess(self.gui, recalculate=False):
 
-            if not self.gui.model.wire.is_valid():
+            if not self.gui.model.wire.valid:
                 self.gui.calculation_status.emit("Calculating Wire Segments … (1/5)")
 
-                if not self.gui.model.calculate_wire(self.progress_callback):
-                    self.on_finished(False)
+                if not self.gui.model.calculate_wire(
+                        self._progress_update.emit  # type: ignore
+                ):
+                    self._on_finished(False)
                     return
 
-                # noinspection PyUnresolvedReferences
-                self._wire_valid.emit()
+                self._wire_valid.emit()  # type: ignore
 
             # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
-            if not self.gui.model.sampling_volume.is_valid():
+            if not self.gui.model.sampling_volume.valid:
                 self.gui.calculation_status.emit("Calculating Sampling Volume … (2/5)")
 
-                if not self.gui.model.calculate_sampling_volume(self.progress_callback):
-                    self.on_finished(False)
+                if not self.gui.model.calculate_sampling_volume(
+                        self._progress_update.emit  # type: ignore
+                ):
+                    self._on_finished(False)
                     return
 
-                # noinspection PyUnresolvedReferences
-                self._sampling_volume_valid.emit()
+                self._sampling_volume_valid.emit()  # type: ignore
 
             # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
-            if not self.gui.model.field.is_valid():
+            if not self.gui.model.field.valid:
                 self.gui.calculation_status.emit("Calculating Field … (3/5)")
 
-                num_cores = self.gui.config.get_int("num_cores")
+                num_cores = self.gui.project.get_int("num_cores")
                 if num_cores == 0:
                     # "Auto" setting
                     num_cores = max(1, cpu_count() - 1)
 
-                success = self.gui.model.calculate_field(self.progress_callback, num_cores)
+                success = self.gui.model.calculate_field(
+                    self._progress_update.emit,  # type: ignore
+                    num_cores
+                )
 
                 if not success:
-                    self.on_finished(False)
+                    self._on_finished(False)
                     return
 
-                # noinspection PyUnresolvedReferences
-                self._field_valid.emit()
+                self._field_valid.emit()  # type: ignore
 
             # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
-            if not self.gui.model.metric.is_valid():
+            if not self.gui.model.metric.valid:
                 self.gui.calculation_status.emit("Calculating Metric … (4/5)")
 
-                if not self.gui.model.calculate_metric(self.progress_callback):
-                    self.on_finished(False)
+                if not self.gui.model.calculate_metric(
+                        self._progress_update.emit  # type: ignore
+                ):
+                    self._on_finished(False)
                     return
 
-                # noinspection PyUnresolvedReferences
-                self._metric_valid.emit()
+                self._metric_valid.emit()  # type: ignore
 
             # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
-            if not self.gui.model.parameters.is_valid():
+            if not self.gui.model.parameters.valid:
                 self.gui.calculation_status.emit("Calculating Parameters … (5/5)")
 
-                if not self.gui.model.calculate_parameters(self.progress_callback):
-                    self.on_finished(False)
+                if not self.gui.model.calculate_parameters(
+                        self._progress_update.emit  # type: ignore
+                ):
+                    self._on_finished(False)
                     return
 
-                # noinspection PyUnresolvedReferences
-                self._parameters_valid.emit()
+                self._parameters_valid.emit()  # type: ignore
 
         # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
-        self.on_finished(True)
+        self._on_finished(True)
 
     # ------------------------------------------------------------------------------------------------------------------
 
-    def on_finished(self, success: bool) -> None:
+    def _on_finished(self, success: bool) -> None:
         """
         Signals that the calculation finished.
 
@@ -160,11 +158,8 @@ class CalculationThread(QThread):
         """
         Debug(self, f".on_finished(success={success})")
 
-        # We cannot directly call this; we won't be able to modify the UI thread (which we'd really like to do somehow):
-        # self.gui.calculation_stopped(success)
-
-        # Instead, we use a signal; however, this results in slightly delayed execution, even after joining this thread;
-        # the execution of  self.gui.on_calculation_exited()  will later be skipped when it sees another thread running.
+        # Firing this signal results in slightly delayed execution, even after joining this thread;
+        # the execution of "on_calculation_exited()" will later be skipped when it sees another thread running.
         self.gui.calculation_exited.emit(success)
 
     # ------------------------------------------------------------------------------------------------------------------
