@@ -16,197 +16,94 @@
 #  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 #  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 
-from typing import Optional, Dict, Callable, List, Union, Any
+from typing import Dict, List, Optional, Union
 import os
+import abc
 import configparser
 import numpy as np
-from magneticalc.Backend_Types import BACKEND_CUDA
 from magneticalc.Debug import Debug
-from magneticalc.Field_Types import B_FIELD
-from magneticalc.Perspective_Presets import Perspective_Presets
-from magneticalc.Version import Version
-from magneticalc.Wire_Presets import Wire_Presets
+from magneticalc.Format import Format
 
 
-def get_jit_enabled() -> bool:
-    """
-    Checks if JIT is enabled (or at least not explicitly disabled).
-
-    @return: Boolean
-    """
-    return (os.environ["NUMBA_DISABLE_JIT"] == "0") if "NUMBA_DISABLE_JIT" in os.environ else True
-
-
-class Config:
+class Config(metaclass=abc.ABCMeta):
     """ Config class. """
 
-    # Formatting settings
-    FloatPrecision = 4
-    CoordinatePrecision = 6
-
-    # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-
-    # Default wire preset
-    DefaultWirePreset = "Straight Line"
-
-    # Default perspective preset
-    DefaultPerspectivePreset = "Isometric"
-
-    # Default configuration
-    Default = {
-        "version"                                   : Version.String,
-        "backend_type"                              : BACKEND_CUDA,
-        "auto_calculation"                          : "True",
-        "num_cores"                                 : "0",
-        "wire_points_base"                          : None,  # Will be set in __init__
-        "wire_stretch"                              : "0.1000, 1.0000, 1.0000",
-        "wire_slicer_limit"                         : "0.0500",
-        "wire_dc"                                   : "1.0000",
-        "wire_close_loop"                           : "True",
-        "rotational_symmetry_count"                 : "30",
-        "rotational_symmetry_radius"                : "1.0000",
-        "rotational_symmetry_axis"                  : "2",
-        "rotational_symmetry_offset"                : "0",
-        "sampling_volume_padding"                   : "1, 1, 1",
-        "sampling_volume_override_padding"          : "False",
-        "sampling_volume_bounding_box"              : "0.000000, 0.000000, 0.000000; 0.000000, 0.000000, 0.000000",
-        "sampling_volume_resolution_exponent"       : "3",
-        "sampling_volume_label_resolution_exponent" : "0",
-        "field_type"                                : B_FIELD,
-        "field_distance_limit"                      : "0.0008",
-        "color_metric"                              : "Log Magnitude",
-        "alpha_metric"                              : "Magnitude",
-        "field_point_scale"                         : "0.0000",
-        "field_arrow_head_scale"                    : "0.7500",
-        "field_arrow_line_scale"                    : "0.7500",
-        "field_boost"                               : "0.0000",
-        "display_field_magnitude_labels"            : "True",
-        "show_wire_segments"                        : "True",
-        "show_wire_points"                          : "True",
-        "show_colored_labels"                       : "True",
-        "show_gauss"                                : "False",
-        "show_coordinate_system"                    : "True",
-        "show_perspective_info"                     : "True",
-        "dark_background"                           : "True",
-        "azimuth"                                   : None,  # Will be set in __init__
-        "elevation"                                 : None,  # Will be set in __init__
-        "scale_factor"                              : "3.0000",
-        "constraint_count"                          : "0"
-    }
+    """ Enables debug output for config access. """
+    DebugAccess = False
 
     def __init__(self) -> None:
         """
         Initializes the configuration.
         """
-        Debug(self, ": Init")
+        Debug(self, ": Init", init=True)
+        self._parser: Optional[configparser.ConfigParser] = None
+        self.filename: str = ""
+        self.synced: bool = False
 
-        # Populate defaults
-        self.Default["wire_points_base"] = Config.points_to_str(
-            Wire_Presets.get_by_id(Config.DefaultWirePreset)["points"]
-        )
-        default_azimuth = Perspective_Presets.get_by_id(Config.DefaultPerspectivePreset)["azimuth"]
-        default_elevation = Perspective_Presets.get_by_id(Config.DefaultPerspectivePreset)["elevation"]
-        self.Default["azimuth"] = f"{float(default_azimuth):.{Config.FloatPrecision}f}"
-        self.Default["elevation"] = f"{float(default_elevation):.{Config.FloatPrecision}f}"
-
-        self._config = configparser.ConfigParser()  # Will be overwritten by L{load()}
-
-        self._synced = False
-        self._changed_callback = None
-
-        self._filename: str = ""
-
-    # ------------------------------------------------------------------------------------------------------------------
-
-    def set_changed_callback(self, config_changed_callback: Callable) -> None:
+    @abc.abstractmethod
+    def on_changed(self) -> None:
         """
-        Sets the callback for any changes to the configuration.
-
-        @param config_changed_callback: Gets called when the filename or the synchronization flag changed
+        Gets called when the project changed.
         """
-        self._changed_callback = config_changed_callback
+        pass
 
-    # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-
-    def get_synced(self) -> bool:
-        """
-        Returns the synchronization state of the current session.
-        """
-        return self._synced
-
-    # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-
-    def set_filename(self, filename: str) -> None:
-        """
-        Sets the filename for the current session.
-
-        @param filename: Filename
-        """
-        self._filename = os.path.join(os.path.dirname(os.path.dirname(__file__)), filename)
-
-        Debug(self, ".set_filename: file://" + self._filename.replace(" ", "%20"))
-
-    def get_filename(self) -> str:
-        """
-        Gets the filename of the current session.
-
-        @return: Filename
-        """
-        return self._filename
-
-    # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-
-    def load(self) -> None:
+    def load_file(self, filename: str, default_config: Dict) -> None:
         """
         Loads the configuration from file.
 
+        @param filename: Filename
+        @param default_config: Default config
         """
-        Debug(self, ".load()")
+        self.filename = Format.absolute_filename(filename)
+        Debug(self, f".load_file(): {Format.filename_uri(self.filename)}")
 
-        self._config = configparser.ConfigParser()
+        self._parser = configparser.ConfigParser()
+        self._parser.read(self.filename)
+        self._parser["DEFAULT"] = default_config
+        self.synced = True
 
-        self._config.read(self._filename)
+        if "User" not in self._parser:
+            Debug(self, ".load_file(): Creating empty User section")
+            self._parser["User"] = {}
+            self.synced = False
 
-        self.set_defaults()
+        if not os.path.isfile(self.filename):
+            Debug(
+                self, f".load_file(): WARNING: File does not exist: {Format.filename_uri(self.filename)}", warning=True
+            )
+            self.synced = False
 
-        self._synced = True
-        if self._changed_callback is not None:
-            self._changed_callback()
+        self.on_changed()
 
-        if not os.path.isfile(self._filename):
-            # Save newly created default file
-            self.save()
-
-    def set_defaults(self) -> None:
-        """
-        Sets the default key-value pairs. Creates empty "User" section if not present.
-        """
-        self._config["DEFAULT"] = Config.Default
-
-        if "User" not in self._config:
-            Debug(self, ".set_defaults(): Creating empty User section")
-            self._config["User"] = {}
-
-    # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-
-    def save(self) -> None:
+    def save_file(self, filename: Optional[str] = None) -> None:
         """
         Saves the configuration to file.
-        """
-        Debug(self, ".save()")
 
-        with open(self._filename, "w") as file:
-            self._config.write(file)
-
-        self._synced = True
-        if self._changed_callback is not None:
-            self._changed_callback()
-
-    def close(self) -> None:
+        @param filename: Filename (optional)
         """
-        Close configuration.
+        assert self._parser is not None, "Not initialized"
+
+        if filename is not None:
+            self.filename = Format.absolute_filename(filename)
+
+        Debug(self, f".save_file(): {Format.filename_uri(self.filename)}")
+
+        with open(self.filename, "w") as file:
+            self._parser.write(file)
+
+        self.synced = True
+        self.on_changed()
+
+    def close_file(self) -> None:
         """
-        Debug(self, ".close()")
+        Closes the configuration.
+        """
+        assert self._parser is not None, "Not initialized"
+
+        Debug(self, ".close_file()")
+        self._parser = None
+        self.filename = ""
+        self.synced = False
 
     # ------------------------------------------------------------------------------------------------------------------
 
@@ -216,124 +113,67 @@ class Config:
 
         @param key: Key
         """
-        if not self._config.remove_option("User", key):
+        assert self._parser is not None, "Not initialized"
+
+        if not self._parser.remove_option("User", key):
             Debug(self, f".remove_key({key}): ERROR: No such key", error=True)
 
     # ------------------------------------------------------------------------------------------------------------------
 
-    def get_bool(self, key: str) -> bool:
+    def set_get_str(self, key: str, value: str) -> str:
         """
-        Gets boolean value, convert from string.
-
-        @param key: Key
-        @return: Value
-        """
-        return self.get_str(key) == "True"
-
-    def get_float(self, key: str) -> float:
-        """
-        Gets float value, convert from string.
-
-        @param key: Key
-        @return: Value
-        """
-        return float(self.get_str(key))
-
-    def get_int(self, key: str) -> int:
-        """
-        Gets integer value, convert from string.
-
-        @param key: Key
-        @return: Value
-        """
-        return int(self.get_str(key))
-
-    def get_points(self, key: str) -> List[List[float]]:
-        """
-        Gets list of 3D points, convert from string.
-
-        @param key: Key
-        @return: List of 3D points
-        """
-        return Config.str_to_points(self.get_str(key))
-
-    def get_point(self, key: str) -> List[float]:
-        """
-        Gets a single 3D point, convert from string.
-
-        @param key: Key
-        @return: Single 3D point
-        """
-        return Config.str_to_point(self.get_str(key))
-
-    def get_str(self, key: str) -> str:
-        """
-        Reads a configuration value. Key must be in "Default" section and may be overridden in "User" section.
-
-        @param key: Key
-        @return: Value
-        """
-        value = self._config.get("User", key)
-        return value
-
-    # ------------------------------------------------------------------------------------------------------------------
-
-    def set_bool(self, key: str, value: bool) -> None:
-        """
-        Sets boolean value, convert to string.
+        If value is not None, writes and returns key-value; otherwise (if value is None), reads and returns key-value.
 
         @param key: Key
         @param value: Value
         """
-        self.set_str(key, "True" if value else "False")
-
-    def set_float(self, key: str, value: float) -> None:
-        """
-        Sets float value, convert to string.
-
-        @param key: Key
-        @param value: Value
-        """
-        self.set_str(key, f"{float(value):.{Config.FloatPrecision}f}")
-
-    def set_int(self, key: str, value: int) -> None:
-        """
-        Sets integer value, convert to string.
-
-        @param key: Key
-        @param value: Value
-        """
-        self.set_str(key, str(int(value)))
-
-    def set_points(self, key: str, value: Union[np.ndarray, List[List[float]]]) -> None:
-        """
-        Sets list of 3D points, convert to string.
-
-        @param key: Key
-        @param value: List of 3D points
-        """
-        self.set_str(key, Config.points_to_str(value))
-
-    def set_point(self, key: str, value: Union[np.ndarray, List[float]]) -> None:
-        """
-        Sets a single 3D point, convert to string.
-
-        @param key: Key
-        @param value: Single 3D point
-        """
-        self.set_str(key, Config.point_to_str(value))
+        if value is None:
+            return self.get_str(key)
+        else:
+            self.set_str(key, value)
+            return value
 
     def set_str(self, key: str, value: str) -> None:
         """
-        Writes a configuration value. Key must be in "Default" section and may be overridden in "User" section.
+        Writes a configuration value.
 
         @param key: Key
         @param value: Value
         """
-        self._config.set("User", key, value)
-        self._synced = False
-        if self._changed_callback is not None:
-            self._changed_callback()
+        assert self._parser is not None, "Not initialized"
+
+        old_value = self._parser.get("User", key, fallback=None)
+        if old_value != value:
+            if self.DebugAccess:
+                if old_value is None:
+                    Debug(self, f".set_str(\"{key}\"): Created key with value \"{value}\"")
+                else:
+                    Debug(self, f".set_str(\"{key}\"): Changing \"{old_value}\" to \"{value}\"")
+            self._parser.set("User", key, value)
+            self.synced = False
+            self.on_changed()
+        else:
+            if self.DebugAccess:
+                Debug(self, f".set_str(\"{key}\"): Unchanged: \"{old_value}\"")
+
+    def get_str(self, key: str) -> str:
+        """
+        Reads a configuration value.
+
+        @param key: Key
+        @return: Value
+        """
+        assert self._parser is not None, "Not initialized"
+
+        value = self._parser.get("User", key, fallback=None)
+        assert value is not None, f"Attempting to read non-existing key: {key}"
+
+        if self.DebugAccess:
+            Debug(self, f".get_str(\"{key}\") = \"{value}\"")
+
+        return value
+
+    # ------------------------------------------------------------------------------------------------------------------
 
     def set_get_bool(self, key: str, value: bool) -> bool:
         """
@@ -348,18 +188,25 @@ class Config:
             self.set_bool(key, value)
             return value
 
-    def set_get_float(self, key: str, value: float) -> float:
+    def set_bool(self, key: str, value: bool) -> None:
         """
-        If value is not None, writes and returns key-value; otherwise (if value is None), reads and returns key-value.
+        Sets boolean value, convert to string.
 
         @param key: Key
         @param value: Value
         """
-        if value is None:
-            return self.get_float(key)
-        else:
-            self.set_float(key, value)
-            return value
+        self.set_str(key, "True" if value else "False")
+
+    def get_bool(self, key: str) -> bool:
+        """
+        Gets boolean value, convert from string.
+
+        @param key: Key
+        @return: Value
+        """
+        return self.get_str(key) == "True"
+
+    # ------------------------------------------------------------------------------------------------------------------
 
     def set_get_int(self, key: str, value: int) -> int:
         """
@@ -374,11 +221,27 @@ class Config:
             self.set_int(key, value)
             return value
 
-    def set_get_points(
-            self,
-            key: str,
-            value: Union[np.ndarray, List[List[float]]]
-    ) -> Union[np.ndarray, List[List[float]]]:
+    def set_int(self, key: str, value: int) -> None:
+        """
+        Sets integer value, convert to string.
+
+        @param key: Key
+        @param value: Value
+        """
+        self.set_str(key, str(int(value)))
+
+    def get_int(self, key: str) -> int:
+        """
+        Gets integer value, convert from string.
+
+        @param key: Key
+        @return: Value
+        """
+        return int(self.get_str(key))
+
+    # ------------------------------------------------------------------------------------------------------------------
+
+    def set_get_float(self, key: str, value: float) -> float:
         """
         If value is not None, writes and returns key-value; otherwise (if value is None), reads and returns key-value.
 
@@ -386,11 +249,31 @@ class Config:
         @param value: Value
         """
         if value is None:
-            return self.get_points(key)
+            return self.get_float(key)
         else:
-            self.set_points(key, value)
+            self.set_float(key, value)
             return value
 
+    def set_float(self, key: str, value: float) -> None:
+        """
+        Sets float value, convert to string.
+
+        @param key: Key
+        @param value: Value
+        """
+        self.set_str(key, Format.float_to_str(value))
+
+    def get_float(self, key: str) -> float:
+        """
+        Gets float value, convert from string.
+
+        @param key: Key
+        @return: Value
+        """
+        return float(self.get_str(key))
+
+    # ------------------------------------------------------------------------------------------------------------------
+
     def set_get_point(
             self,
             key: str,
@@ -408,7 +291,31 @@ class Config:
             self.set_point(key, value)
             return value
 
-    def set_get_str(self, key: str, value: str) -> str:
+    def set_point(self, key: str, value: Union[np.ndarray, List[float]]) -> None:
+        """
+        Sets a single 3D point, convert to string.
+
+        @param key: Key
+        @param value: Single 3D point
+        """
+        self.set_str(key, Format.point_to_str(value))
+
+    def get_point(self, key: str) -> List[float]:
+        """
+        Gets a single 3D point, convert from string.
+
+        @param key: Key
+        @return: Single 3D point
+        """
+        return Format.str_to_point(self.get_str(key))
+
+    # ------------------------------------------------------------------------------------------------------------------
+
+    def set_get_points(
+            self,
+            key: str,
+            value: Union[np.ndarray, List[List[float]]]
+    ) -> Union[np.ndarray, List[List[float]]]:
         """
         If value is not None, writes and returns key-value; otherwise (if value is None), reads and returns key-value.
 
@@ -416,66 +323,59 @@ class Config:
         @param value: Value
         """
         if value is None:
-            return self.get_str(key)
+            return self.get_points(key)
         else:
-            self.set_str(key, value)
+            self.set_points(key, value)
             return value
 
-    # ------------------------------------------------------------------------------------------------------------------
-
-    @staticmethod
-    def point_to_str(point: Union[np.ndarray, List[float]]) -> str:
+    def set_points(self, key: str, value: Union[np.ndarray, List[List[float]]]) -> None:
         """
-        Converts a single 3D point to string.
+        Sets list of 3D points, convert to string.
 
-        @param point: Single 3D point
-        @return: String
+        @param key: Key
+        @param value: List of 3D points
         """
-        return \
-            f"{point[0]:+0.0{Config.CoordinatePrecision}f}, " \
-            f"{point[1]:+0.0{Config.CoordinatePrecision}f}, " \
-            f"{point[2]:+0.0{Config.CoordinatePrecision}f}"
+        self.set_str(key, Format.points_to_str(value))
 
-    @staticmethod
-    def points_to_str(points: Union[np.ndarray, List[List[float]]]) -> str:
+    def get_points(self, key: str) -> List[List[float]]:
         """
-        Converts list of 3D points to string.
+        Gets list of 3D points, convert from string.
 
-        @param points: List of 3D points
-        @return: String
-        """
-        return "; ".join([Config.point_to_str(point) for point in points])
-
-    @staticmethod
-    def str_to_point(str_point: str) -> Union[np.ndarray, List[float]]:
-        """
-        Converts string to single 3D point.
-
-        @param str_point: String
-        @return: Single 3D point
-        """
-        return [float(coord) for coord in str_point.split(",")]
-
-    @staticmethod
-    def str_to_points(str_points: str) -> Union[np.ndarray, List[List[float]]]:
-        """
-        Converts string to list of 3D points.
-
-        @param str_points: String
+        @param key: Key
         @return: List of 3D points
         """
-        return [
-            Config.str_to_point(str_point) for str_point in str_points.split(";")
-            if str_point != ""  # Ignoring stray semicolon at the end of the line
-        ]
+        return Format.str_to_points(self.get_str(key))
 
     # ------------------------------------------------------------------------------------------------------------------
 
+    def set_get_dict(self, prefix: str, suffix: str, types: Dict, values: Optional[Dict]) -> Dict:
+        """
+        If "values" is None, reads and returns all key-values in "types".
+        Note: The actual keys saved in the configuration file are prefixed with "prefix", and suffixed with "suffix".
+
+        If "values" is not None, writes and returns all key-values in "types".
+        Note: In this case, "values" must have the same keys as "types".
+
+        @param prefix: Prefix
+        @param suffix: Suffix
+        @param types: Key:Type (Dictionary)
+        @param values: Key:Value (Dictionary)
+        """
+        if values is None:
+            result = {}
+            for _key, _type in types.items():
+                result[_key] = self.get_generic(prefix + _key + suffix, _type)
+            return result
+        else:
+            for _key, _type in types.items():
+                self.set_generic(prefix + _key + suffix, _type, values[_key])
+            return values
+
     def get_generic(
             self,
             _key: str,
             _type: str
-    ) -> Union[bool, float, int, str, np.ndarray, List[float], List[List[float]]]:
+    ) -> Union[str, bool, int, float, np.ndarray, List[float], List[List[float]]]:
         """
         Gets some "_key"-"_value" of data type "_type".
 
@@ -501,7 +401,7 @@ class Config:
             self,
             _key: str,
             _type: str,
-            _value: Union[bool, float, int, str, np.ndarray, List[float], List[List[float]]]
+            _value: Union[str, bool, int, float, np.ndarray, List[float], List[List[float]]]
     ) -> None:
         """
         Sets some "_key"-"_value" of data type "_type".
@@ -523,26 +423,3 @@ class Config:
             set_func[_type](_key, _value)
         else:
             raise TypeError("Invalid type")
-
-    def set_get_dict(self, prefix: str, suffix: str, types: Dict, values: Optional[Dict]) -> Dict:
-        """
-        If "values" is None, reads and returns all key-values in "types".
-        Note: The actual keys saved in the configuration file are prefixed with "prefix", and suffixed with "suffix".
-
-        If "values" is not None, writes and returns all key-values in "types".
-        Note: In this case, "values" must have the same keys as "types".
-
-        @param prefix: Prefix
-        @param suffix: Suffix
-        @param types: Key:Type (Dictionary)
-        @param values: Key:Value (Dictionary)
-        """
-        if values is None:
-            result = {}
-            for _key, _type in types.items():
-                result[_key] = self.get_generic(prefix + _key + suffix, _type)
-            return result
-        else:
-            for _key, _type in types.items():
-                self.set_generic(prefix + _key + suffix, _type, values[_key])
-            return values
