@@ -29,28 +29,28 @@ from magneticalc.QLabel2 import QLabel2
 from magneticalc.SamplingVolume_Widget import SamplingVolume_Widget
 from magneticalc.QSliderFloat import QSliderFloat
 from magneticalc.Theme import Theme
-from magneticalc.VispyCanvas import VispyCanvas
+from magneticalc.VisPyCanvas import VisPyCanvas
 
 
 class Display_Widget(QGroupBox2):
     """ Display_Widget class. """
 
     # Slider limits
-    FieldArrowHeadScaleMinimum = 0
-    FieldArrowHeadScaleMaximum = 1
-    FieldArrowHeadScaleStep = 1 / VispyCanvas.FieldArrowHeadSize
-    FieldArrowLineScaleMinimum = 0
-    FieldArrowLineScaleMaximum = 1
+    FieldArrowHeadScaleMin = 0
+    FieldArrowHeadScaleMax = 1
+    FieldArrowHeadScaleStep = 1 / VisPyCanvas.FieldArrowHeadSize
+    FieldArrowLineScaleMin = 0
+    FieldArrowLineScaleMax = 1
     FieldArrowLineScaleStep = 1 / 10
-    FieldPointScaleMinimum = 0
-    FieldPointScaleMaximum = 1
-    FieldPointScaleStep = 1 / VispyCanvas.FieldPointSize
-    FieldBoostMinimum = 0
-    FieldBoostMaximum = 1
+    FieldPointScaleMin = 0
+    FieldPointScaleMax = 1
+    FieldPointScaleStep = 1 / VisPyCanvas.FieldPointSize
+    FieldBoostMin = 0
+    FieldBoostMax = 1
     FieldBoostStep = 1 / 20
 
     # Warn about displaying an excessive number of field labels
-    ExcessiveFieldLabelThreshold = 250
+    ExcessiveFieldLabelsThreshold = 250
 
     def __init__(
             self,
@@ -62,20 +62,19 @@ class Display_Widget(QGroupBox2):
         @param gui: GUI
         """
         QGroupBox2.__init__(self, "Display")
-        Debug(self, ": Init")
+        Debug(self, ": Init", init=True)
         self.gui = gui
 
         # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
         self.addLayout(QIconLabel("Point Scale", "fa.circle"))
-        self.field_point_scale_slider = QSliderFloat(Qt.Horizontal)
-        self.field_point_scale_slider.set_range_step(
-            self.FieldPointScaleMinimum,
-            self.FieldPointScaleMaximum,
-            self.FieldPointScaleStep
+        self.field_point_scale_slider = QSliderFloat(
+            orientation=Qt.Horizontal,
+            minimum=self.FieldPointScaleMin,
+            maximum=self.FieldPointScaleMax,
+            step=self.FieldPointScaleStep
         )
-        # noinspection PyUnresolvedReferences
-        self.field_point_scale_slider.valueChanged.connect(
+        self.field_point_scale_slider.valueChanged.connect(  # type: ignore
             lambda: self.set_field_point_scale(self.field_point_scale_slider.get_value())
         )
         self.addWidget(self.field_point_scale_slider)
@@ -90,27 +89,25 @@ class Display_Widget(QGroupBox2):
         field_arrow_scale_layout_right = QVBoxLayout()
 
         field_arrow_scale_layout_left.addWidget(QLabel2("Head:", expand=False))
-        self.field_arrow_head_scale_slider = QSliderFloat(Qt.Horizontal)
-        self.field_arrow_head_scale_slider.set_range_step(
-            self.FieldArrowHeadScaleMinimum,
-            self.FieldArrowHeadScaleMaximum,
-            self.FieldArrowHeadScaleStep
+        self.field_arrow_head_scale_slider = QSliderFloat(
+            orientation=Qt.Horizontal,
+            minimum=self.FieldArrowHeadScaleMin,
+            maximum=self.FieldArrowHeadScaleMax,
+            step=self.FieldArrowHeadScaleStep
         )
-        # noinspection PyUnresolvedReferences
-        self.field_arrow_head_scale_slider.valueChanged.connect(
+        self.field_arrow_head_scale_slider.valueChanged.connect(  # type: ignore
             lambda: self.set_field_arrow_head_scale(self.field_arrow_head_scale_slider.get_value())
         )
         field_arrow_scale_layout_right.addWidget(self.field_arrow_head_scale_slider)
 
         field_arrow_scale_layout_left.addWidget(QLabel2("Line:", expand=False))
-        self.field_arrow_line_scale_slider = QSliderFloat(Qt.Horizontal)
-        self.field_arrow_line_scale_slider.set_range_step(
-            self.FieldArrowLineScaleMinimum,
-            self.FieldArrowLineScaleMaximum,
-            self.FieldArrowLineScaleStep
+        self.field_arrow_line_scale_slider = QSliderFloat(
+            orientation=Qt.Horizontal,
+            minimum=self.FieldArrowLineScaleMin,
+            maximum=self.FieldArrowLineScaleMax,
+            step=self.FieldArrowLineScaleStep
         )
-        # noinspection PyUnresolvedReferences
-        self.field_arrow_line_scale_slider.valueChanged.connect(
+        self.field_arrow_line_scale_slider.valueChanged.connect(  # type: ignore
             lambda: self.set_field_arrow_line_scale(self.field_arrow_line_scale_slider.get_value())
         )
         field_arrow_scale_layout_right.addWidget(self.field_arrow_line_scale_slider)
@@ -125,14 +122,13 @@ class Display_Widget(QGroupBox2):
         self.addWidget(QHLine())
 
         self.addLayout(QIconLabel("Field Boost", "fa.adjust"))
-        self.field_boost_slider = QSliderFloat(Qt.Horizontal)
-        self.field_boost_slider.set_range_step(
-            self.FieldBoostMinimum,
-            self.FieldBoostMaximum,
-            self.FieldBoostStep
+        self.field_boost_slider = QSliderFloat(
+            orientation=Qt.Horizontal,
+            minimum=self.FieldBoostMin,
+            maximum=self.FieldBoostMax,
+            step=self.FieldBoostStep
         )
-        # noinspection PyUnresolvedReferences
-        self.field_boost_slider.valueChanged.connect(
+        self.field_boost_slider.valueChanged.connect(  # type: ignore
             lambda: self.set_field_boost(self.field_boost_slider.get_value())
         )
         self.addWidget(self.field_boost_slider)
@@ -143,8 +139,7 @@ class Display_Widget(QGroupBox2):
 
         self.addLayout(QIconLabel("Field Labels", "fa.tags"))
         self.display_field_magnitude_labels_checkbox = QCheckBox(" Display Magnitude")
-        # noinspection PyUnresolvedReferences
-        self.display_field_magnitude_labels_checkbox.toggled.connect(
+        self.display_field_magnitude_labels_checkbox.toggled.connect(  # type: ignore
             lambda: self.set_display_field_magnitude_labels(self.display_field_magnitude_labels_checkbox.isChecked())
         )
         self.addWidget(self.display_field_magnitude_labels_checkbox)
@@ -163,37 +158,101 @@ class Display_Widget(QGroupBox2):
         total_labels_layout.addWidget(self.total_labels_label)
         self.addLayout(total_labels_layout)
 
-        # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-
-        self.reinitialize()
-
-    def reinitialize(self) -> None:
+    def reload(self) -> None:
         """
-        Re-initializes the widget.
+        Reloads the widget.
         """
-        Debug(self, ".reinitialize()")
+        Debug(self, ".reload()", refresh=True)
 
         self.blockSignals(True)
 
-        self.field_point_scale_slider.setValue(self.gui.config.get_float("field_point_scale"))
-        self.field_arrow_head_scale_slider.setValue(self.gui.config.get_float("field_arrow_head_scale"))
-        self.field_arrow_line_scale_slider.setValue(self.gui.config.get_float("field_arrow_line_scale"))
-        self.field_boost_slider.setValue(self.gui.config.get_float("field_boost"))
+        self.field_point_scale_slider.setValue(self.gui.project.get_float("field_point_scale"))
+        self.field_arrow_head_scale_slider.setValue(self.gui.project.get_float("field_arrow_head_scale"))
+        self.field_arrow_line_scale_slider.setValue(self.gui.project.get_float("field_arrow_line_scale"))
+        self.field_boost_slider.setValue(self.gui.project.get_float("field_boost"))
         self.display_field_magnitude_labels_checkbox.setChecked(
-            self.gui.config.get_bool("display_field_magnitude_labels")
+            self.gui.project.get_bool("display_field_magnitude_labels")
         )
 
         self.blockSignals(False)
 
         self.update()
 
-    def set_enabled(self, enabled: bool) -> None:
+    def update(self) -> None:
         """
-        Enables/disables this widget.
+        Updates this widget.
+        """
+        Debug(self, ".update()", refresh=True)
 
-        @param enabled: Enabled state
+        self.update_labels()
+        self.update_controls()
+
+    def update_labels(self) -> None:
         """
-        self.setEnabled(enabled)
+        Updates the labels.
+        """
+        Debug(self, ".update_labels()", refresh=True)
+
+        if self.gui.model.sampling_volume.valid:
+            n = self.gui.model.sampling_volume.get_labels_count()
+            color = Theme.FailureColor if n > self.ExcessiveFieldLabelsThreshold else Theme.MainColor
+            self.total_labels_label.setText(str(n))
+            self.total_labels_label.setStyleSheet(f"color: {color}; font-style: italic;")
+        else:
+            self.total_labels_label.setText("N/A")
+            self.total_labels_label.setStyleSheet(f"color: {Theme.LiteColor}; font-style: italic;")
+
+    def update_controls(self) -> None:
+        """
+        Updates the field label resolution combobox.
+        """
+        Debug(self, ".update_controls()", refresh=True)
+
+        # Possible field label resolution values: Less than or equal to sampling volume resolution
+        sampling_volume_resolution = self.gui.model.sampling_volume.get_resolution()
+        label_resolution_options_dict = {
+            key: value for key, value in SamplingVolume_Widget.ResolutionOptionsDict.items()
+            if np.power(2.0, value) <= sampling_volume_resolution
+        }
+
+        self.blockSignals(True)
+
+        # Repopulate field label resolution combobox
+        if self.field_label_resolution_combobox_connection is not None:
+            self.field_label_resolution_combobox.currentIndexChanged.disconnect(  # type: ignore
+                self.field_label_resolution_combobox_connection
+            )
+        self.field_label_resolution_combobox.clear()
+        for i, value in enumerate(label_resolution_options_dict.keys()):
+            self.field_label_resolution_combobox.addItem(str(value))
+
+        def connection():
+            """ Sets field label resolution. """
+            self.set_field_label_resolution(
+                label_resolution_options_dict.get(
+                    self.field_label_resolution_combobox.currentText(),
+                    0
+                )
+            )
+        self.field_label_resolution_combobox_connection = connection
+        self.field_label_resolution_combobox.currentIndexChanged.connect(connection)  # type: ignore
+
+        # Set default field label resolution if it is not available anymore
+        target = self.gui.project.get_int("sampling_volume_label_resolution_exponent")
+        if target not in label_resolution_options_dict.values():
+            Debug(self, f": WARNING: Invalid: sampling_volume_label_resolution_exponent = {target}", warning=True)
+            self.gui.project.set_int(
+                "sampling_volume_label_resolution_exponent",
+                next(iter(label_resolution_options_dict.items()))[1]  # First value from combobox
+            )
+
+        # Select the field label resolution
+        target = self.gui.project.get_int("sampling_volume_label_resolution_exponent")
+        for i, value in enumerate(label_resolution_options_dict.values()):
+            if value == target:
+                self.field_label_resolution_combobox.setCurrentIndex(i)
+
+        self.blockSignals(False)
 
     # ------------------------------------------------------------------------------------------------------------------
 
@@ -206,7 +265,7 @@ class Display_Widget(QGroupBox2):
         if self.signalsBlocked():
             return
 
-        self.gui.config.set_float("field_point_scale", value)
+        self.gui.project.set_float("field_point_scale", value)
         self.gui.redraw()
 
     def set_field_arrow_head_scale(self, value: float) -> None:
@@ -218,7 +277,7 @@ class Display_Widget(QGroupBox2):
         if self.signalsBlocked():
             return
 
-        self.gui.config.set_float("field_arrow_head_scale", value)
+        self.gui.project.set_float("field_arrow_head_scale", value)
         self.gui.redraw()
 
     def set_field_arrow_line_scale(self, value: float) -> None:
@@ -230,7 +289,7 @@ class Display_Widget(QGroupBox2):
         if self.signalsBlocked():
             return
 
-        self.gui.config.set_float("field_arrow_line_scale", value)
+        self.gui.project.set_float("field_arrow_line_scale", value)
         self.gui.redraw()
 
     def set_field_boost(self, value: float) -> None:
@@ -242,7 +301,7 @@ class Display_Widget(QGroupBox2):
         if self.signalsBlocked():
             return
 
-        self.gui.config.set_float("field_boost", value)
+        self.gui.project.set_float("field_boost", value)
         self.gui.redraw()
 
     # ------------------------------------------------------------------------------------------------------------------
@@ -256,7 +315,7 @@ class Display_Widget(QGroupBox2):
         if self.signalsBlocked():
             return
 
-        self.gui.config.set_bool("display_field_magnitude_labels", value)
+        self.gui.project.set_bool("display_field_magnitude_labels", value)
 
         self.prevent_excessive_field_labels(choice=True)
 
@@ -275,7 +334,7 @@ class Display_Widget(QGroupBox2):
 
         self.gui.sidebar_left.sampling_volume_widget.set_sampling_volume(_label_resolution_exponent_=value)
 
-        # Note: L{prevent_excessive_field_labels(choice=False)} will be called by L{Model.on_sampling_volume_valid()}.
+        # Note: "prevent_excessive_field_labels()" will be called by "Model.on_sampling_volume_valid()".
 
         self.gui.redraw()
 
@@ -285,7 +344,7 @@ class Display_Widget(QGroupBox2):
         """
         Debug(self, ".disable_field_labels()")
 
-        self.gui.config.set_bool("display_field_magnitude_labels", False)
+        self.gui.project.set_bool("display_field_magnitude_labels", False)
 
         previous_signals_blocked = self.signalsBlocked()
         self.blockSignals(True)
@@ -300,21 +359,23 @@ class Display_Widget(QGroupBox2):
         """
         Debug(self, f".prevent_excessive_field_labels(choice={choice})")
 
-        if not self.gui.config.get_bool("display_field_magnitude_labels"):
+        if not self.gui.project.get_bool("display_field_magnitude_labels"):
             return
 
-        if not self.gui.model.sampling_volume.is_valid():
+        if not self.gui.model.sampling_volume.valid:
             return
 
-        if not self.gui.model.sampling_volume.get_labels_count() > self.ExcessiveFieldLabelThreshold:
+        if not self.gui.model.sampling_volume.get_labels_count() > self.ExcessiveFieldLabelsThreshold:
             return
 
         if choice:
             text = (
-                "You are about to display an excessive number of field labels. "
+                "You are about to display an excessive number of field labels.\n"
                 "This will be very slow and cannot be interrupted.\n\n"
-                "DO YOU WANT TO DISPLAY FIELD LABELS ANYWAY?\n\n"
+
+                "DO YOU WANT TO DISPLAY FIELD LABELS ANYWAY?\n"
                 "Choosing 'No' will disable field labels immediately.\n\n"
+
                 "Please consider the following before choosing 'Yes':\n"
                 "– Save your work first.\n"
                 "– Decrease sampling volume resolution.\n"
@@ -327,7 +388,7 @@ class Display_Widget(QGroupBox2):
                 buttons=QMessageBox.Yes | QMessageBox.No,
                 default_button=QMessageBox.No
             )
-            if messagebox.choice == QMessageBox.No:
+            if not messagebox.user_accepted or messagebox.choice == QMessageBox.No:
                 self.disable_field_labels()
 
         else:
@@ -337,7 +398,9 @@ class Display_Widget(QGroupBox2):
             text = (
                 "Field labels were disabled automatically because\n"
                 "an excessive number of field labels was detected.\n\n"
-                "You may manually re-enable field labels after all calculations have finished."
+
+                "You can manually re-enable field labels\n"
+                "after all calculations have finished."
             )
             QMessageBox2(
                 title="Excessive Number Of Field Labels",
@@ -346,83 +409,3 @@ class Display_Widget(QGroupBox2):
                 buttons=QMessageBox.Ok,
                 default_button=QMessageBox.Ok
             )
-
-    # ------------------------------------------------------------------------------------------------------------------
-
-    def update(self) -> None:
-        """
-        Updates this widget.
-        """
-        Debug(self, ".update()")
-
-        self.update_labels()
-        self.update_controls()
-
-    def update_labels(self) -> None:
-        """
-        Updates the labels.
-        """
-        Debug(self, ".update_labels()")
-
-        if self.gui.model.sampling_volume.is_valid():
-            n = self.gui.model.sampling_volume.get_labels_count()
-            color = Theme.FailureColor if n > self.ExcessiveFieldLabelThreshold else Theme.LiteColor
-            self.total_labels_label.setText(str(n))
-            self.total_labels_label.setStyleSheet(f"color: {color}; font-style: italic;")
-        else:
-            self.total_labels_label.setText("N/A")
-            self.total_labels_label.setStyleSheet(f"color: {Theme.LiteColor}; font-style: italic;")
-
-    def update_controls(self) -> None:
-        """
-        Updates the field label resolution combobox.
-        """
-        Debug(self, ".update_controls()")
-
-        # Possible field label resolution values: Less than or equal to sampling volume resolution
-        sampling_volume_resolution = self.gui.model.sampling_volume.get_resolution()
-        label_resolution_options_dict = {
-            key: value for key, value in SamplingVolume_Widget.ResolutionOptionsDict.items()
-            if np.power(2.0, value) <= sampling_volume_resolution
-        }
-
-        self.blockSignals(True)
-
-        # Re-populate field label resolution combobox
-        if self.field_label_resolution_combobox_connection is not None:
-            # noinspection PyUnresolvedReferences
-            self.field_label_resolution_combobox.currentIndexChanged.disconnect(
-                self.field_label_resolution_combobox_connection
-            )
-        self.field_label_resolution_combobox.clear()
-        for i, value in enumerate(label_resolution_options_dict.keys()):
-            self.field_label_resolution_combobox.addItem(str(value))
-
-        def connection():
-            """ Sets field label resolution. """
-            self.set_field_label_resolution(
-                label_resolution_options_dict.get(
-                    self.field_label_resolution_combobox.currentText(),
-                    0
-                )
-            )
-        self.field_label_resolution_combobox_connection = connection
-        # noinspection PyUnresolvedReferences
-        self.field_label_resolution_combobox.currentIndexChanged.connect(connection)
-
-        # Set default field label resolution if it is not available anymore
-        target = self.gui.config.get_int("sampling_volume_label_resolution_exponent")
-        if target not in label_resolution_options_dict.values():
-            Debug(self, f": WARNING: Invalid: sampling_volume_label_resolution_exponent = {target}", warning=True)
-            self.gui.config.set_int(
-                "sampling_volume_label_resolution_exponent",
-                next(iter(label_resolution_options_dict.items()))[1]  # First value from combobox
-            )
-
-        # Select the field label resolution
-        target = self.gui.config.get_int("sampling_volume_label_resolution_exponent")
-        for i, value in enumerate(label_resolution_options_dict.values()):
-            if value == target:
-                self.field_label_resolution_combobox.setCurrentIndex(i)
-
-        self.blockSignals(False)
