@@ -20,58 +20,56 @@ from typing import Tuple, Callable
 import numpy as np
 from numba import jit, prange, set_num_threads
 from magneticalc.Assert_Dialog import Assert_Dialog
-from magneticalc.Backend_Types import BACKEND_JIT, BACKEND_CUDA
+from magneticalc.Backend_Types import BACKEND_TYPE_JIT, BACKEND_TYPE_CUDA, get_jit_enabled
 from magneticalc.Backend_CUDA import Backend_CUDA
 from magneticalc.Backend_JIT import Backend_JIT
 from magneticalc.ConditionalDecorator import ConditionalDecorator
-from magneticalc.Config import get_jit_enabled
 from magneticalc.Debug import Debug
-from magneticalc.Field_Types import A_FIELD, B_FIELD, Field_Types_Str_Map
+from magneticalc.Field_Types import FIELD_TYPE_A, FIELD_TYPE_B
 from magneticalc.SamplingVolume import SamplingVolume
+from magneticalc.Validatable import Validatable, require_valid, validator
 from magneticalc.Wire import Wire
 
 
-class Field:
+class Field(Validatable):
     """ Field class. """
 
-    def __init__(self, backend_type: int, field_type: int, distance_limit: float, length_scale: float) -> None:
+    def __init__(self) -> None:
         """
         Initializes an empty field.
+        """
+        Validatable.__init__(self)
+        Debug(self, ": Init", init=True)
+
+        self._total_calculations: int = 0
+        self._total_skipped_calculations: int = 0
+        self._vectors: np.ndarray = np.array([])
+
+        self._backend_type = 0
+        self._field_type = 0
+        self._distance_limit = 0
+        self._length_scale = 0
+
+    def set(
+            self,
+            backend_type: int,
+            field_type: int,
+            distance_limit: float,
+            length_scale: float
+    ):
+        """
+        Sets the parameters.
 
         @param backend_type: Backend type
         @param field_type: Field type
         @param distance_limit: Distance limit (mitigating divisions by zero)
         @param length_scale: Length scale (m)
         """
-        Debug(self, ": Init")
-
         self._backend_type = backend_type
         self._field_type = field_type
         self._distance_limit = distance_limit
         self._length_scale = length_scale
 
-        self._total_calculations: int = 0
-        self._total_skipped_calculations: int = 0
-        self._vectors: np.ndarray = np.array([])
-
-        self._valid = False
-
-    def is_valid(self) -> bool:
-        """
-        Indicates valid data for display.
-
-        @return: True if data is valid for display, False otherwise
-        """
-        return self._valid
-
-    def invalidate(self) -> None:
-        """
-        Resets data, hiding from display.
-        """
-        Debug(self, ".invalidate()")
-
-        self._valid = False
-
     def get_type(self) -> int:
         """
         Gets field type.
@@ -87,51 +85,47 @@ class Field:
         @param show_gauss: Enable to show Gauss instead of Tesla
         @return: Field units, field factor
         """
-        Assert_Dialog(self._field_type in Field_Types_Str_Map, "Attempting to get units for invalid field type")
-
         if show_gauss:
             return {
-                A_FIELD: "Gs·m",    # Gauss · meter
-                B_FIELD: "Gs"       # Gauss
+                FIELD_TYPE_A: "Gs·m",    # Gauss · meter
+                FIELD_TYPE_B: "Gs"       # Gauss
             }.get(self._field_type, ""), 1e4
         else:
             return {
-                A_FIELD: "T·m",     # Tesla · meter
-                B_FIELD: "T"        # Tesla
+                FIELD_TYPE_A: "T·m",     # Tesla · meter
+                FIELD_TYPE_B: "T"        # Tesla
             }.get(self._field_type, ""), 1e0
 
+    @require_valid
     def get_vectors(self) -> np.ndarray:
         """
         Gets field vectors. (The selected field type determined which field was calculated.)
 
         @return: Ordered list of 3D vectors (field vectors & corresponding sampling volume points have the same indices)
         """
-        Assert_Dialog(self.is_valid(), "Accessing invalidated field")
-
         return self._vectors
 
+    @require_valid
     def get_total_calculations(self) -> int:
         """
         Gets total number of calculations.
 
         @return: Total number of calculations
         """
-        Assert_Dialog(self.is_valid(), "Accessing invalidated field")
-
         return self._total_calculations
 
+    @require_valid
     def get_total_skipped_calculations(self) -> int:
         """
         Gets total number of skipped calculations.
 
         @return: Total number of skipped calculations
         """
-        Assert_Dialog(self.is_valid(), "Accessing invalidated field")
-
         return self._total_skipped_calculations
 
     # ------------------------------------------------------------------------------------------------------------------
 
+    @validator
     def recalculate(
             self,
             wire: Wire,
@@ -150,21 +144,10 @@ class Field:
         """
         Debug(self, ".recalculate()")
 
-        self._valid = False
-
         # Compute the current elements.
         current_elements = wire.get_elements()
 
-        # Default to JIT backend if CUDA backend is selected but not available.
-        if self._backend_type == BACKEND_CUDA:
-            if not Backend_CUDA.is_available():
-                Debug(
-                    self, f".recalculate(): WARNING: CUDA backend not available, defaulting to JIT backend",
-                    warning=True
-                )
-                self._backend_type = BACKEND_JIT
-
-        if self._backend_type == BACKEND_JIT:
+        if self._backend_type == BACKEND_TYPE_JIT:
 
             # Initialize Biot-Savart JIT backend
             backend = Backend_JIT(
@@ -182,7 +165,7 @@ class Field:
             set_num_threads(num_cores)
             backend_result = backend.get_result()
 
-        elif self._backend_type == BACKEND_CUDA:
+        elif self._backend_type == BACKEND_TYPE_CUDA:
 
             # Initialize Biot-Savart CUDA backend
             backend = Backend_CUDA(
@@ -219,8 +202,6 @@ class Field:
             Assert_Dialog(False, "ERROR: Unexpected number of calculations – Backend seems to be buggy")
             return False
 
-        self._valid = True
-
         return True
 
     # ------------------------------------------------------------------------------------------------------------------
@@ -236,7 +217,7 @@ class Field:
             magnitude_limit: float
     ) -> Tuple[np.ndarray, np.ndarray]:
         """
-        Returns the field arrow parameters needed by L{VispyCanvas}.
+        Returns the field arrow parameters needed by L{VisPyCanvas}.
 
         @param sampling_volume_points: Sampling volume points
         @param field_vectors: Field vectors
