@@ -16,24 +16,35 @@
 #  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 #  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 
-from typing import Optional
+from magneticalc.Assert_Dialog import Assert_Dialog
 
 
-A_FIELD = 0
-B_FIELD = 1
+""" Field type: A-Field. """
+FIELD_TYPE_A = 0
+
+""" Field type: B-Field. """
+FIELD_TYPE_B = 1
 
 
-Field_Types_Str_Map = {
-    A_FIELD: "A-Field",
-    B_FIELD: "B-Field"
+""" Map of field types to names. """
+Field_Types_Names_Map = {
+    FIELD_TYPE_A    :   "A-Field (Vector Potential)",
+    FIELD_TYPE_B    :   "B-Field (Flux Density)"
 }
 
+""" Default field type. """
+Field_Type_Default = FIELD_TYPE_A
 
-def field_type_to_str(field_type: int) -> Optional[str]:
+
+def field_type_safe(field_type: int) -> int:
     """
-    Converts a field type to a field string.
+    A valid field type is passed through, but an invalid field type converts to the default type.
 
     @param field_type: Field type
-    @return: Field string, or None if field type is invalid
+    @return: Safe field type
     """
-    return Field_Types_Str_Map.get(field_type, None)
+    if field_type in Field_Types_Names_Map:
+        return field_type
+    else:
+        Assert_Dialog(False, "Invalid field type, using fallback type")
+        return Field_Type_Default
