@@ -20,15 +20,14 @@ from __future__ import annotations
 from typing import Optional
 from functools import partial
 from PyQt5.QtWidgets import QHBoxLayout, QVBoxLayout, QSizePolicy
-from PyQt5.QtWidgets import QButtonGroup, QRadioButton, QCheckBox
+from PyQt5.QtWidgets import QButtonGroup, QCheckBox, QRadioButton
 from magneticalc.QDoubleSpinBox2 import QDoubleSpinBox2
 from magneticalc.QGroupBox2 import QGroupBox2
 from magneticalc.QHLine import QHLine
 from magneticalc.QIconLabel import QIconLabel
 from magneticalc.QLabel2 import QLabel2
 from magneticalc.Debug import Debug
-from magneticalc.Field import Field
-from magneticalc.Field_Types import A_FIELD, B_FIELD
+from magneticalc.Field_Types import Field_Types_Names_Map
 from magneticalc.Metric import Metric
 from magneticalc.ModelAccess import ModelAccess
 from magneticalc.Theme import Theme
@@ -53,7 +52,7 @@ class Field_Widget(QGroupBox2):
         @param gui: GUI
         """
         QGroupBox2.__init__(self, "Field")
-        Debug(self, ": Init")
+        Debug(self, ": Init", init=True)
         self.gui = gui
 
         # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -63,23 +62,17 @@ class Field_Widget(QGroupBox2):
         field_type_layout_left = QVBoxLayout()
         field_type_layout_right = QVBoxLayout()
 
-        field_type_a_radiobutton = QRadioButton(" A-Field (Vector Potential)")
-        field_type_layout_left.addWidget(field_type_a_radiobutton)
-        self.field_type_a_checkbox = QCheckBox(" Cached")
-        self.field_type_a_checkbox.setSizePolicy(QSizePolicy.Fixed, QSizePolicy.Expanding)
-        self.field_type_a_checkbox.setEnabled(False)
-        field_type_layout_right.addWidget(self.field_type_a_checkbox)
-
-        field_type_b_radiobutton = QRadioButton(" B-Field (Flux Density)")
-        field_type_layout_left.addWidget(field_type_b_radiobutton)
-        self.field_type_b_checkbox = QCheckBox(" Cached")
-        self.field_type_b_checkbox.setSizePolicy(QSizePolicy.Fixed, QSizePolicy.Expanding)
-        self.field_type_b_checkbox.setEnabled(False)
-        field_type_layout_right.addWidget(self.field_type_b_checkbox)
-
         self.field_type_group = QButtonGroup()
-        self.field_type_group.addButton(field_type_a_radiobutton)
-        self.field_type_group.addButton(field_type_b_radiobutton)
+        field_type_radiobuttons = {}
+        self.field_type_checkboxes = {}
+        for field_type, field_name in Field_Types_Names_Map.items():
+            field_type_radiobuttons[field_type] = QRadioButton(" " + field_name)
+            field_type_layout_left.addWidget(field_type_radiobuttons[field_type])
+            self.field_type_checkboxes[field_type] = QCheckBox(" Cached")
+            self.field_type_checkboxes[field_type].setSizePolicy(QSizePolicy.Fixed, QSizePolicy.Expanding)
+            self.field_type_checkboxes[field_type].setEnabled(False)
+            field_type_layout_right.addWidget(self.field_type_checkboxes[field_type])
+            self.field_type_group.addButton(field_type_radiobuttons[field_type])
 
         field_type_layout = QHBoxLayout()
         field_type_layout.addLayout(field_type_layout_left)
@@ -122,30 +115,56 @@ class Field_Widget(QGroupBox2):
         total_skipped_calculations_layout.addWidget(self.total_skipped_calculations_label)
         self.addLayout(total_skipped_calculations_layout)
 
-        # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-
-        self.reinitialize()
-
-    def reinitialize(self) -> None:
+    def reload(self) -> None:
         """
-        Re-initializes the widget.
+        Reloads the widget.
         """
-        Debug(self, ".reinitialize()")
+        Debug(self, ".reload()", refresh=True)
 
         self.blockSignals(True)
 
+        field_type = self.gui.project.get_int("field_type")
         for i, button in enumerate(self.field_type_group.buttons()):
-            button.setChecked(i == self.gui.config.get_int("field_type"))
+            button.setChecked(i == field_type)
 
-        self.distance_limit_spinbox.setValue(self.gui.config.get_float("field_distance_limit"))
+        self.distance_limit_spinbox.setValue(self.gui.project.get_float("field_distance_limit"))
 
         self.blockSignals(False)
 
-        # Initially load field from configuration
-        self.set_field(recalculate=False, invalidate_self=False)
+        # Initially load field from project
+        self.set_field(recalculate=False, invalidate=False)
 
         self.update()
 
+    def update(self) -> None:
+        """
+        Updates this widget.
+        """
+        Debug(self, ".update()", refresh=True)
+
+        self.update_labels()
+        self.update_controls()
+
+    def update_labels(self) -> None:
+        """
+        Updates the labels.
+        """
+        if self.gui.model.field.valid:
+            self.total_calculations_label.setText(str(self.gui.model.field.get_total_calculations()))
+            self.total_skipped_calculations_label.setText(str(self.gui.model.field.get_total_skipped_calculations()))
+        else:
+            self.total_calculations_label.setText("N/A")
+            self.total_skipped_calculations_label.setText("N/A")
+
+    def update_controls(self) -> None:
+        """
+        Updates the controls.
+        """
+        for field_type in Field_Types_Names_Map:
+            self.field_type_checkboxes[field_type].setChecked(self.gui.model.get_valid_field(field_type) is not None)
+
+        self.indicate_valid(self.gui.model.field.valid)
+
     # ------------------------------------------------------------------------------------------------------------------
 
     def on_field_type_changed(self, field_type: int, checked: bool) -> None:
@@ -161,7 +180,7 @@ class Field_Widget(QGroupBox2):
         if not checked:
             return
 
-        self.set_field(_field_type_=field_type, allow_cache=True)
+        self.set_field(_field_type_=field_type, invalidate=False)
 
     # ------------------------------------------------------------------------------------------------------------------
 
@@ -169,9 +188,8 @@ class Field_Widget(QGroupBox2):
             self,
             _field_type_: Optional[int] = None,
             _distance_limit_: Optional[float] = None,
-            recalculate: bool = True,
-            invalidate_self: bool = True,
-            allow_cache: bool = False
+            invalidate: bool = True,
+            recalculate: bool = True
     ) -> None:
         """
         Sets the field. This will replace the currently set field in the model.
@@ -179,9 +197,8 @@ class Field_Widget(QGroupBox2):
 
         @param _field_type_: Field type
         @param _distance_limit_: Distance limit
-        @param recalculate: Enable to trigger final re-calculation
-        @param invalidate_self: Enable to invalidate the old field before setting a new one
-        @param allow_cache: Enable to select a field from the cache if it is available (based on the field type)
+        @param invalidate: Enable to invalidate this model hierarchy level
+        @param recalculate: Enable to trigger final recalculation
         """
         if self.signalsBlocked():
             return
@@ -190,55 +207,14 @@ class Field_Widget(QGroupBox2):
 
         with ModelAccess(self.gui, recalculate):
 
-            field_type = self.gui.config.set_get_int("field_type", _field_type_)
-
-            if allow_cache:
-                field = self.gui.model.get_valid_field(field_type)
-            else:
-                field = None
-
-            if field is not None:
-                Debug(self, ".set_field(): Using cached field")
-            else:
-                backend_type = self.gui.config.get_int("backend_type")
-                distance_limit = self.gui.config.set_get_float("field_distance_limit", _distance_limit_)
-                field = Field(backend_type, field_type, distance_limit, Metric.LengthScale)
+            field_type = self.gui.project.set_get_int("field_type", _field_type_)
+            backend_type = self.gui.project.get_int("backend_type")
+            distance_limit = self.gui.project.set_get_float("field_distance_limit", _distance_limit_)
 
             self.gui.model.set_field(
-                field=field,
-                invalidate_self=invalidate_self and not allow_cache,
+                invalidate=invalidate,
+                backend_type=backend_type,
+                field_type=field_type,
+                distance_limit=distance_limit,
+                length_scale=Metric.LengthScale
             )
-
-    # ------------------------------------------------------------------------------------------------------------------
-
-    def update(self) -> None:
-        """
-        Updates this widget.
-        """
-        Debug(self, ".update()")
-
-        self.update_labels()
-        self.update_controls()
-
-    def update_labels(self) -> None:
-        """
-        Updates the labels.
-        """
-        if self.gui.model.field.is_valid():
-            self.total_calculations_label.setText(str(self.gui.model.field.get_total_calculations()))
-            self.total_skipped_calculations_label.setText(str(self.gui.model.field.get_total_skipped_calculations()))
-        else:
-            self.total_calculations_label.setText("N/A")
-            self.total_skipped_calculations_label.setText("N/A")
-
-    def update_controls(self) -> None:
-        """
-        Updates the controls.
-        """
-        a_field_available = self.gui.model.get_valid_field(A_FIELD) is not None
-        b_field_available = self.gui.model.get_valid_field(B_FIELD) is not None
-
-        self.field_type_a_checkbox.setChecked(a_field_available)
-        self.field_type_b_checkbox.setChecked(b_field_available)
-
-        self.indicate_valid(self.gui.model.field is not None and self.gui.model.field.is_valid())
