@@ -22,9 +22,10 @@ from typing import Dict, List, Callable, Tuple
 import numpy as np
 from numba import jit, prange
 from magneticalc.Assert_Dialog import Assert_Dialog
+from magneticalc.Backend_Types import get_jit_enabled
 from magneticalc.ConditionalDecorator import ConditionalDecorator
-from magneticalc.Config import get_jit_enabled
 from magneticalc.Debug import Debug
+from magneticalc.Validatable import Validatable, require_valid, validator
 
 
 @ConditionalDecorator(get_jit_enabled(), jit, nopython=True, parallel=False)
@@ -112,7 +113,7 @@ def metric_divergence(neighborhood_vectors: np.ndarray, dL: float, polarity: int
 
 
 @ConditionalDecorator(get_jit_enabled(), jit, nopython=True, parallel=False)
-def color_map_divergent(color_normalized: float) -> Tuple[float, float, float]:
+def color_map_divergent(color_normalized: float) -> np.ndarray:
     """
     Maps normalized value to color, divergent.
 
@@ -127,7 +128,7 @@ def color_map_divergent(color_normalized: float) -> Tuple[float, float, float]:
 
 
 @ConditionalDecorator(get_jit_enabled(), jit, nopython=True, parallel=False)
-def color_map_cyclic(color_normalized: float) -> Tuple[float, float, float]:
+def color_map_cyclic(color_normalized: float) -> np.ndarray:
     """
     Maps normalized value to color, cyclic.
 
@@ -144,7 +145,7 @@ def color_map_cyclic(color_normalized: float) -> Tuple[float, float, float]:
     return np.array([r, g, b])
 
 
-class Metric:
+class Metric(Validatable):
     """
     Provides different metrics, used for mapping some field vector properties to some color and alpha range.
     """
@@ -153,43 +154,39 @@ class Metric:
     LengthScale = 1e-2  # m  (1 cm)
 
     # Minimum argument limit for logarithmic scaling
-    LogNormMinimum = 1e-12
+    LogNormMin = 1e-12
 
     # ------------------------------------------------------------------------------------------------------------------
 
-    def __init__(self, color_preset: Dict, alpha_preset: Dict) -> None:
+    def __init__(self) -> None:
         """
+        Initializes an empty metric.
+
         This class holds a pair of metric presets.
         Using these metric presets, colors (including alpha channel) and field limits are calculated.
-
-        @param color_preset: Color metric preset (dictionary)
-        @param alpha_preset: Alpha metric preset (dictionary)
         """
-        Debug(self, ": Init")
+        Validatable.__init__(self)
+        Debug(self, ": Init", init=True)
 
-        self._color_preset = color_preset
-        self._alpha_preset = alpha_preset
+        self._color_preset = {}
+        self._alpha_preset = {}
 
         self._colors: np.ndarray = np.array([])
         self._limits: Dict = {}
 
-        self._valid = False
-
-    def is_valid(self) -> bool:
+    def set(
+            self,
+            color_preset: Dict,
+            alpha_preset: Dict
+    ) -> None:
         """
-        Indicates valid data for display.
+        Sets the parameters.
 
-        @return: True if data is valid for display, False otherwise
+        @param color_preset: Color metric preset (dictionary)
+        @param alpha_preset: Alpha metric preset (dictionary)
         """
-        return self._valid
-
-    def invalidate(self) -> None:
-        """
-        Resets data, hiding from display.
-        """
-        Debug(self, ".invalidate()")
-
-        self._valid = False
+        self._color_preset = color_preset
+        self._alpha_preset = alpha_preset
 
     def get_color_preset(self) -> Dict:
         """
@@ -207,24 +204,22 @@ class Metric:
         """
         return self._alpha_preset
 
+    @require_valid
     def get_colors(self) -> np.ndarray:
         """
         Returns calculated colors.
 
         @return: Ordered list of color values (4-tuples)
         """
-        Assert_Dialog(self.is_valid(), "Accessing invalidated metric")
-
         return self._colors
 
+    @require_valid
     def get_limits(self) -> Dict:
         """
         Returns calculated limits.
 
         @return: Dictionary
         """
-        Assert_Dialog(self.is_valid(), "Accessing invalidated metric")
-
         return self._limits
 
     # ------------------------------------------------------------------------------------------------------------------
@@ -363,6 +358,7 @@ class Metric:
 
         return colors
 
+    @validator
     def recalculate(
             self,
             sampling_volume: SamplingVolume,  # type: ignore
@@ -379,8 +375,6 @@ class Metric:
         """
         Debug(self, ".recalculate()")
 
-        self._valid = False
-
         n = len(field.get_vectors())
 
         progress_callback(0)
@@ -447,7 +441,7 @@ class Metric:
             Assert_Dialog(not self._color_preset["is_angle"], "Logarithmic angles don't make any sense")
 
             # Adjust range for logarithm (out-of-range values will be clipped in the loop below)
-            color_norm_min_ = max(color_norm_min, Metric.LogNormMinimum)    # avoiding ValueError: min must be positive
+            color_norm_min_ = max(color_norm_min, Metric.LogNormMin)    # avoiding ValueError: min must be positive
             color_norm_max_ = max(color_norm_min_, color_norm_max)          # avoiding ValueError: min must be <= max
         else:
             # Linear normalization
@@ -461,7 +455,7 @@ class Metric:
             Assert_Dialog(not self._alpha_preset["is_angle"], "Logarithmic angles don't make any sense")
 
             # Adjust range for logarithm (out-of-range values will be clipped in the loop below)
-            alpha_norm_min_ = max(alpha_norm_min, Metric.LogNormMinimum)    # avoiding ValueError: min must be positive
+            alpha_norm_min_ = max(alpha_norm_min, Metric.LogNormMin)    # avoiding ValueError: min must be positive
             alpha_norm_max_ = max(alpha_norm_min_, alpha_norm_max)          # avoiding ValueError: min must be <= max
         else:
             # Linear normalization
@@ -496,8 +490,6 @@ class Metric:
 
         progress_callback(100)
 
-        self._valid = True
-
         return True
 
     # ------------------------------------------------------------------------------------------------------------------
