@@ -76,15 +76,14 @@ class Metric_Widget(QGroupBox2):
         @param gui: GUI
         """
         QGroupBox2.__init__(self, "Metric")
-        Debug(self, ": Init")
+        Debug(self, ": Init", init=True)
         self.gui = gui
 
         self.addLayout(QIconLabel("Color", "fa.tint"))
         self.color_metric_combobox = QComboBox()
         for i, preset in enumerate(Metric_Presets.List):
             self.color_metric_combobox.addItem(preset["id"])
-        # noinspection PyUnresolvedReferences
-        self.color_metric_combobox.currentIndexChanged.connect(
+        self.color_metric_combobox.currentIndexChanged.connect(  # type: ignore
             lambda: self.set_metric(_color_preset_=Metric_Presets.get_by_id(self.color_metric_combobox.currentText()))
         )
         self.addWidget(self.color_metric_combobox)
@@ -114,8 +113,7 @@ class Metric_Widget(QGroupBox2):
             # Using angle metric for alpha transparency is discouraged and is not available in the combobox anymore.
             if not preset["is_angle"]:
                 self.alpha_metric_combobox.addItem(preset["id"])
-        # noinspection PyUnresolvedReferences
-        self.alpha_metric_combobox.currentIndexChanged.connect(
+        self.alpha_metric_combobox.currentIndexChanged.connect(  # type: ignore
             lambda: self.set_metric(_alpha_preset_=Metric_Presets.get_by_id(self.alpha_metric_combobox.currentText()))
         )
         self.addWidget(self.alpha_metric_combobox)
@@ -134,87 +132,36 @@ class Metric_Widget(QGroupBox2):
         self.alpha_metric_limits_layout.addWidget(self.alpha_metric_max_label)
         self.addWidget(self.alpha_metric_limits_widget)
 
-        self.reinitialize()
-
-    def reinitialize(self) -> None:
+    def reload(self) -> None:
         """
-        Re-initializes the widget.
+        Reloads the widget.
         """
-        Debug(self, ".reinitialize()")
+        Debug(self, ".reload()", refresh=True)
 
         self.blockSignals(True)
 
+        color_metric = self.gui.project.get_str("color_metric")
         for i, preset in enumerate(Metric_Presets.List):
-            if preset["id"] == self.gui.config.get_str("color_metric"):
+            if preset["id"] == color_metric:
                 self.color_metric_combobox.setCurrentIndex(i)
 
+        alpha_metric = self.gui.project.get_str("alpha_metric")
         for i, preset in enumerate(Metric_Presets.List):
-            if preset["id"] == self.gui.config.get_str("alpha_metric"):
+            if preset["id"] == alpha_metric:
                 self.alpha_metric_combobox.setCurrentIndex(i)
 
         self.blockSignals(False)
 
-        # Initially load metric from configuration
-        self.set_metric(recalculate=False, update_labels=False, invalidate_self=False)
+        # Initially load metric from project
+        self.set_metric(recalculate=False, update_labels=False, invalidate=False)
 
         self.update()
 
-    # ------------------------------------------------------------------------------------------------------------------
-
-    def set_metric(
-            self,
-            _color_preset_: Optional[Dict] = None,
-            _alpha_preset_: Optional[Dict] = None,
-            recalculate: bool = True,
-            update_labels: bool = True,
-            invalidate_self: bool = True
-    ) -> None:
-        """
-        Sets the metric. This will overwrite the currently set metric in the model.
-        Any underscored parameter may be left set to None in order to load its default value.
-
-        @param _color_preset_: Color metric preset (parameters, see Metric module)
-        @param _alpha_preset_: Alpha metric preset (parameters, see Metric module)
-        @param recalculate: Enable to trigger final re-calculation
-        @param update_labels: Enable to update metric labels
-        @param invalidate_self: Enable to invalidate the old metric before setting a new one
-        """
-        if self.signalsBlocked():
-            return
-
-        Debug(self, ".set_metric()")
-
-        with ModelAccess(self.gui, recalculate):
-
-            # Note: Not using self.gui.config.write_read_str here because we're translating between strings and presets
-
-            if _color_preset_ is None:
-                color_preset = Metric_Presets.get_by_id(self.gui.config.get_str("color_metric"))
-            else:
-                self.gui.config.set_str("color_metric", _color_preset_["id"])
-                color_preset = _color_preset_
-
-            if _alpha_preset_ is None:
-                alpha_preset = Metric_Presets.get_by_id(self.gui.config.get_str("alpha_metric"))
-            else:
-                self.gui.config.set_str("alpha_metric", _alpha_preset_["id"])
-                alpha_preset = _alpha_preset_
-
-            self.gui.model.set_metric(
-                Metric(color_preset, alpha_preset),
-                invalidate_self=invalidate_self
-            )
-
-            if update_labels:
-                self.update_labels()
-
-    # ------------------------------------------------------------------------------------------------------------------
-
     def update(self) -> None:
         """
         Updates the widget.
         """
-        Debug(self, ".update()")
+        Debug(self, ".update()", refresh=True)
 
         self.update_labels()
         self.update_controls()
@@ -223,13 +170,13 @@ class Metric_Widget(QGroupBox2):
         """
         Updates the labels.
         """
-        Debug(self, ".update_labels()")
+        Debug(self, ".update_labels()", refresh=True)
 
-        if self.gui.model.metric.is_valid():
+        if self.gui.model.metric.valid:
 
             limits = self.gui.model.metric.get_limits()
 
-            show_gauss = self.gui.config.get_bool("show_gauss")
+            show_gauss = self.gui.project.get_bool("show_gauss")
             field_units, field_factor = self.gui.model.field.get_units(show_gauss=show_gauss)
 
             if self.gui.model.metric.get_color_preset()["is_angle"]:
@@ -329,6 +276,53 @@ class Metric_Widget(QGroupBox2):
         """
         Updates the controls.
         """
-        Debug(self, ".update_controls()")
+        Debug(self, ".update_controls()", refresh=True)
+        self.indicate_valid(self.gui.model.metric.valid)
 
-        self.indicate_valid(self.gui.model.metric is not None and self.gui.model.metric.is_valid())
+    # ------------------------------------------------------------------------------------------------------------------
+
+    def set_metric(
+            self,
+            _color_preset_: Optional[Dict] = None,
+            _alpha_preset_: Optional[Dict] = None,
+            invalidate: bool = True,
+            recalculate: bool = True,
+            update_labels: bool = True
+    ) -> None:
+        """
+        Sets the metric. This will overwrite the currently set metric in the model.
+        Any underscored parameter may be left set to None in order to load its default value.
+
+        @param _color_preset_: Color metric preset (parameters, see Metric module)
+        @param _alpha_preset_: Alpha metric preset (parameters, see Metric module)
+        @param invalidate: Enable to invalidate this model hierarchy level
+        @param recalculate: Enable to trigger final recalculation
+        @param update_labels: Enable to update metric labels
+        """
+        if self.signalsBlocked():
+            return
+
+        Debug(self, ".set_metric()")
+
+        with ModelAccess(self.gui, recalculate):
+
+            if _color_preset_ is None:
+                color_preset = Metric_Presets.get_by_id(self.gui.project.get_str("color_metric"))
+            else:
+                self.gui.project.set_str("color_metric", _color_preset_["id"])
+                color_preset = _color_preset_
+
+            if _alpha_preset_ is None:
+                alpha_preset = Metric_Presets.get_by_id(self.gui.project.get_str("alpha_metric"))
+            else:
+                self.gui.project.set_str("alpha_metric", _alpha_preset_["id"])
+                alpha_preset = _alpha_preset_
+
+            self.gui.model.set_metric(
+                invalidate=invalidate,
+                color_preset=color_preset,
+                alpha_preset=alpha_preset
+            )
+
+            if update_labels:
+                self.update_labels()
