@@ -17,11 +17,16 @@
 #  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 
 from __future__ import annotations
-from typing import Optional, Dict, Any, Callable, List
+from typing import Optional, Dict, Callable
 from sty import fg
 from magneticalc.Debug import Debug
-from magneticalc.Field_Types import field_type_to_str
+from magneticalc.Field import Field
+from magneticalc.Field_Types import FIELD_TYPE_A, FIELD_TYPE_B
+from magneticalc.Metric import Metric
 from magneticalc.ModelAccess import ModelAccess
+from magneticalc.Parameters import Parameters
+from magneticalc.SamplingVolume import SamplingVolume
+from magneticalc.Wire import Wire
 
 
 class Model:
@@ -30,16 +35,14 @@ class Model:
 
     The model maintains a strict hierarchy of dependencies:  parameters > metric > field > sampling volume > wire
     When a lower module's data changed, all higher modules are invalidated (i.e. have their calculation results reset).
-
-    Note: The model maintains a cache for fields of different types, with at most one selected ("active") field.
-    Property decorators maintain coherency between the selected field type and the contents of the field cache.
-    The currently selected field is accessed using the L{field} property.
-    Invalidating the currently selected field also invalidates all the other cached fields.
     """
 
     # Used by L{Debug}
     DebugColor = fg.yellow
 
+    """ Enable to debug calls to L{invalidate()}. """
+    DebugInvalidate = False
+
     def __init__(
             self,
             gui: GUI  # type: ignore
@@ -49,114 +52,67 @@ class Model:
 
         @param gui: GUI
         """
-        Debug(self, ": Init")
+        Debug(self, ": Init", init=True)
         self.gui = gui
 
-        # Set in L{set_wire} via L{Wire_Widget}
-        self.wire: Optional[Wire] = None  # type: ignore
+        self.wire = Wire()
+        self.sampling_volume = SamplingVolume()
+        self.metric = Metric()
+        self.parameters = Parameters()
 
-        # Set in L{set_sampling_volume} via L{SamplingVolume_Widget}
-        self.sampling_volume: Optional[SamplingVolume] = None  # type: ignore
-
-        # Set in L{set_metric} via L{Metric_Widget}
-        self.metric: Optional[Metric] = None  # type: ignore
-
-        # Set in L{set_parameters} via L{Parameters_Widget}
-        self.parameters: Optional[Parameters] = None  # type: ignore
-
-        # Field cache and currently selected field:
-
-        # Set in L{set_field} via L{Field_Widget}
-        self._selected_field_type: Optional[int] = None
-
-        # Set in L{set_field} via L{Field_Widget}
-        self._field_cache: Dict[
-            int,
-            Field  # type: ignore
-        ] = {}
+        self._field_type_select: int = FIELD_TYPE_A
+        self._field_cache: Dict[int, Field] = {
+            FIELD_TYPE_A: Field(),
+            FIELD_TYPE_B: Field(),
+        }
 
     @property
-    def field(
-            self
-    ) -> Optional[Field]:  # type: ignore
+    def field_type_select(self) -> int:
         """
-        Returns the currently selected field if it is cached.
+        @return: Currently selected field type
+        """
+        return self._field_type_select
 
-        @return: Field if cached, None otherwise
+    @field_type_select.setter
+    def field_type_select(self, field_type: int) -> None:
         """
-        return self._field_cache.get(self._selected_field_type, None) if self._selected_field_type is not None else None
-
-    @field.setter
-    def field(
-            self,
-            field: Field  # type: ignore
-    ) -> None:
-        """
-        Sets the currently selected field.
-
-        @param field: Field
-        """
-        self._selected_field_type = field.get_type()
-        self._field_cache[self._selected_field_type] = field
-
-    def get_valid_field(
-            self,
-            field_type: int
-    ) -> Optional[Field]:  # type: ignore
-        """
-        Gets a field if it is cached and valid.
+        Sets the currently selected field type.
 
         @param field_type: Field type
-        @return: Field if cached and valid, None otherwise
         """
-        field = self._field_cache.get(field_type, None)
-        return (field if field.is_valid() else None) if field is not None else None
+        Debug(self, f".field_type_select = {field_type}")
+        self._field_type_select = field_type
+
+    @property
+    def field(self) -> Field:
+        """
+        @return: Currently selected field
+        """
+        return self._field_cache[self.field_type_select]
+
+    def get_valid_field(self, field_type: int) -> Optional[Field]:
+        """
+        Gets a field by type if the field is valid.
+
+        @param field_type: Field type
+        @return: Field if valid, None otherwise
+        """
+        field = self._field_cache[field_type]
+        return field if field.valid else None
 
     # ------------------------------------------------------------------------------------------------------------------
 
-    @staticmethod
-    def safe_valid(obj: Any) -> bool:
+    @property
+    def valid(self) -> bool:
         """
-        Safely checks the validity state of a possibly uninitialized object within the model.
-        """
-        return obj.is_valid() if obj is not None else False
-
-    def __str__(self) -> str:
-        """
-        Returns the model validity state as a string.
-
-        @return: String
-        """
-        return ", ".join(self.get_summary(valid=True))
-
-    def get_summary(self, valid: bool) -> List[str]:
-        """
-        Gets a list of (in)valid object names.
-
-        @param valid: True returns names of valid objects, False returns names of invalid objects
-        @return: List of strings
-        """
-        valid_map = {
-            "Wire"              : self.safe_valid(self.wire),
-            "Sampling Volume"   : self.safe_valid(self.sampling_volume),
-            "Field"             : self.safe_valid(self.field),
-            "Metric"            : self.safe_valid(self.metric),
-            "Parameters"        : self.safe_valid(self.parameters)
-        }
-        return [name for name, is_valid in valid_map.items() if valid == is_valid]
-
-    def is_valid(self) -> bool:
-        """
-        Indicates valid data for display.
-
-        @return: True if data is valid for display, False otherwise
+        @return: True if model is valid, False otherwise
         """
         return \
-            self.safe_valid(self.wire) and \
-            self.safe_valid(self.sampling_volume) and \
-            self.safe_valid(self.field) and \
-            self.safe_valid(self.metric) and \
-            self.safe_valid(self.parameters)
+            self.wire.valid and \
+            self.sampling_volume.valid and \
+            self.field.valid and \
+            self.metric.valid and \
+            self.parameters.valid
 
     def invalidate(
             self,
@@ -164,162 +120,172 @@ class Model:
             do_sampling_volume: bool = False,
             do_field: bool = False,
             do_metric: bool = False,
-            do_parameters: bool = False
+            do_parameters: bool = False,
+            do_all: bool = False
     ) -> None:
         """
-        Invalidates multiple modules at once, in descending order of hierarchy.
+        Invalidates multiple hierarchy levels at once, in descending order of hierarchy.
 
         @param do_wire: Enable to invalidate wire
         @param do_sampling_volume: Enable to invalidate sampling volume
-        @param do_field: Enable to invalidate all fields (the currently selected field and all the other cached fields)
+        @param do_field: Enable to invalidate all fields
         @param do_metric: Enable to invalidate metric
         @param do_parameters: Enable to invalidate parameters
+        @param do_all: Enable to invalidate all hierarchy levels
         """
-        Debug(self, ".invalidate()")
+        if self.DebugInvalidate:
+            subject = {
+                "wire": do_wire,
+                "sampling_volume": do_sampling_volume,
+                "field": do_field,
+                "metric": do_metric,
+                "parameters": do_parameters,
+                "all": do_all
+            }
+            string = ", ".join([name for name, condition in subject.items() if condition])
+            Debug(self, f".invalidate({string})")
 
-        if do_parameters:
-            if self.parameters is not None:
-                if self.parameters.is_valid():
-                    self.parameters.invalidate()
-                    self.on_parameters_invalid()
+        if do_all or do_parameters:
+            if self.parameters.valid:
+                self.parameters.valid = False
+                self.on_parameters_invalid()
 
-        if do_metric:
-            if self.metric is not None:
-                if self.metric.is_valid():
-                    self.metric.invalidate()
-                    self.on_metric_invalid()
+        if do_all or do_metric:
+            if self.metric.valid:
+                self.metric.valid = False
+                self.on_metric_invalid()
 
-        if do_field:
-            did_field_invalidation = False
-            for field_type, field in self._field_cache.items():
-                if field.is_valid():
-                    field.invalidate()
-                    Debug(self, f".invalidate(): Invalidated {field_type_to_str(field_type)}")
-                    did_field_invalidation = True
-            if did_field_invalidation:
+        if do_all or do_field:
+            valid_fields = [field for field_type, field in self._field_cache.items() if field.valid]
+            if any(valid_fields):
+                for field in valid_fields:
+                    field.valid = False
                 self.on_field_invalid()
 
-        if do_sampling_volume:
-            if self.sampling_volume is not None:
-                if self.sampling_volume.is_valid():
-                    self.sampling_volume.invalidate()
-                    self.on_sampling_volume_invalid()
+        if do_all or do_sampling_volume:
+            if self.sampling_volume.valid:
+                self.sampling_volume.valid = False
+                self.on_sampling_volume_invalid()
 
-        if do_wire:
-            if self.wire is not None:
-                if self.wire.is_valid():
-                    self.wire.invalidate()
-                    self.on_wire_invalid()
+        if do_all or do_wire:
+            if self.wire.valid:
+                self.wire.valid = False
+                self.on_wire_invalid()
 
     def set_wire(
             self,
-            wire: Wire,  # type: ignore
-            invalidate_self: bool
+            invalidate: bool,
+            *args,
+            **kwargs
     ) -> None:
         """
         Sets the wire.
 
-        @param wire: Wire
-        @param invalidate_self: Enable to invalidate the old object before setting a new one
+        @param invalidate: Enable to invalidate this model hierarchy level
         """
         Debug(self, ".set_wire()")
 
         with ModelAccess(self.gui, recalculate=False):
 
             self.invalidate(
-                do_wire=invalidate_self,
+                do_wire=invalidate,
                 do_sampling_volume=True,
                 do_field=True,
                 do_metric=True,
                 do_parameters=True
             )
-            self.wire = wire
+            self.wire.set(*args, **kwargs)
 
     def set_sampling_volume(
             self,
-            sampling_volume: SamplingVolume,  # type: ignore
-            invalidate_self: bool
+            invalidate: bool,
+            *args,
+            **kwargs
     ) -> None:
         """
         Sets the sampling volume.
 
-        @param sampling_volume: Sampling volume
-        @param invalidate_self: Enable to invalidate the old object before setting a new one
+        @param invalidate: Enable to invalidate this model hierarchy level
         """
         Debug(self, ".set_sampling_volume()")
 
         with ModelAccess(self.gui, recalculate=False):
 
             self.invalidate(
-                do_sampling_volume=invalidate_self,
+                do_sampling_volume=invalidate,
                 do_field=True,
                 do_metric=True,
                 do_parameters=True
             )
-            self.sampling_volume = sampling_volume
+            self.sampling_volume.set(*args, **kwargs)
 
     def set_field(
             self,
-            field: Field,  # type: ignore
-            invalidate_self: bool
+            invalidate: bool,
+            field_type: int,
+            *args,
+            **kwargs
     ) -> None:
         """
         Sets the field.
 
-        @param field: Field
-        @param invalidate_self: Enable to invalidate the old object (including the cache) before setting a new one
+        @param invalidate: Enable to invalidate this model hierarchy level
+        @param field_type: Field type
         """
         Debug(self, ".set_field()")
 
         with ModelAccess(self.gui, recalculate=False):
 
+            self.field_type_select = field_type
+
             self.invalidate(
-                do_field=invalidate_self,
+                do_field=invalidate,
                 do_metric=True,
                 do_parameters=True
             )
-            self.field = field
+
+            self.field.set(field_type=field_type, *args, **kwargs)
 
     def set_metric(
             self,
-            metric: Metric,  # type: ignore
-            invalidate_self: bool
+            invalidate: bool,
+            *args,
+            **kwargs
     ) -> None:
         """
         Sets the metric.
 
-        @param metric: Metric
-        @param invalidate_self: Enable to invalidate the old object before setting a new one
+        @param invalidate: Enable to invalidate this model hierarchy level
         """
         Debug(self, ".set_metric()")
 
         with ModelAccess(self.gui, recalculate=False):
 
             self.invalidate(
-                do_metric=invalidate_self,
+                do_metric=invalidate,
                 do_parameters=True
             )
-            self.metric = metric
+            self.metric.set(*args, **kwargs)
 
     def set_parameters(
             self,
-            parameters: Parameters,  # type: ignore
-            invalidate_self: bool
+            invalidate: bool,
+            *args,
+            **kwargs
     ) -> None:
         """
         Sets the parameters.
 
-        @param parameters: Parameters
-        @param invalidate_self: Enable to invalidate the old object before setting a new one
+        @param invalidate: Enable to invalidate this model hierarchy level
         """
         Debug(self, ".set_parameters()")
 
         with ModelAccess(self.gui, recalculate=False):
 
             self.invalidate(
-                do_parameters=invalidate_self
+                do_parameters=invalidate
             )
-            self.parameters = parameters
+            self.parameters.set(*args, **kwargs)
 
     # ------------------------------------------------------------------------------------------------------------------
 
@@ -331,11 +297,7 @@ class Model:
         @return: True if successful, False if interrupted
         """
         Debug(self, ".calculate_wire()")
-
-        assert self.wire is not None, "Attempting to calculate non-existing Wire"
-
         self.invalidate(do_sampling_volume=True, do_field=True, do_metric=True)
-
         return self.wire.recalculate(progress_callback)
 
     def calculate_sampling_volume(self, progress_callback: Callable) -> bool:
@@ -346,9 +308,6 @@ class Model:
         @return: True if successful, False if interrupted
         """
         Debug(self, ".calculate_sampling_volume()")
-
-        assert self.sampling_volume is not None, "Attempting to calculate non-existing SamplingVolume"
-
         self.invalidate(do_field=True, do_metric=True)
         return self.sampling_volume.recalculate(progress_callback)
 
@@ -361,9 +320,6 @@ class Model:
         @return: True if successful, False if interrupted
         """
         Debug(self, f".calculate_field(num_cores={num_cores})")
-
-        assert self.field is not None, "Attempting to calculate non-existing Field"
-
         self.invalidate(do_metric=True)
         return self.field.recalculate(self.wire, self.sampling_volume, progress_callback, num_cores)
 
@@ -375,9 +331,6 @@ class Model:
         @return: True (currently non-interruptable)
         """
         Debug(self, ".calculate_metric()")
-
-        assert self.metric is not None, "Attempting to calculate non-existing Metric"
-
         return self.metric.recalculate(self.sampling_volume, self.field, progress_callback)
 
     def calculate_parameters(self, progress_callback: Callable) -> bool:
@@ -388,9 +341,6 @@ class Model:
         @return: True (currently non-interruptable)
         """
         Debug(self, ".calculate_parameters()")
-
-        assert self.parameters is not None, "Attempting to calculate non-existing Parameters"
-
         return self.parameters.recalculate(self.wire, self.sampling_volume, self.field, progress_callback)
 
     # ------------------------------------------------------------------------------------------------------------------
@@ -402,7 +352,7 @@ class Model:
         Debug(self, ".on_wire_valid()")
 
         self.gui.sidebar_left.wire_widget.update()
-        self.gui.menu.update_wire_menu()
+        self.gui.menu.wire_menu.update()
 
     def on_sampling_volume_valid(self) -> None:
         """
@@ -425,14 +375,13 @@ class Model:
     def on_metric_valid(self) -> None:
         """
         Gets called when the metric was successfully calculated.
+
+        Note: Field label creation is triggered on-demand inside L{VisPyCanvas.redraw()}, not here.
         """
         Debug(self, ".on_metric_valid()")
 
         self.gui.sidebar_right.metric_widget.update()
 
-        # The field labels are now created on-demand inside VispyCanvas.redraw()
-        # self.gui.vispy_canvas.create_field_labels()
-
     def on_parameters_valid(self) -> None:
         """
         Gets called when the parameters were successfully calculated.
@@ -450,7 +399,7 @@ class Model:
         Debug(self, ".on_wire_invalid()")
 
         self.gui.sidebar_left.wire_widget.update()
-        self.gui.menu.update_wire_menu()
+        self.gui.menu.wire_menu.update()
 
     def on_sampling_volume_invalid(self) -> None:
         """
