@@ -36,12 +36,12 @@ class OverridePadding_Dialog(QDialog2):
             gui: GUI  # type: ignore
     ) -> None:
         """
-        Initializes "Override Padding" dialog.
+        Initializes the dialog.
 
         @param gui: GUI
         """
         QDialog2.__init__(self, title="Override Padding", width=420)
-        Debug(self, ": Init")
+        Debug(self, ": Init", init=True)
         self.gui = gui
 
         self.addWidget(
@@ -51,7 +51,7 @@ class OverridePadding_Dialog(QDialog2):
         self.addLayout(QHBoxLayout2(QLabel2("Units:", expand=False), QLabel2("cm", bold=True)))
         self.addSpacing(16)
 
-        bounding_box = self.gui.config.get_points("sampling_volume_bounding_box")
+        bounding_box = self.gui.project.get_points("sampling_volume_bounding_box")
         self.bounds_min_spinbox = [
             QSpinBox2(self.BoundsRange[0], self.BoundsRange[1], bounding_box[0][i], self.validate)
             for i in range(3)
@@ -80,6 +80,6 @@ class OverridePadding_Dialog(QDialog2):
         """
         Validates the bounding box values and enables/disables the "Apply" button accordingly.
         """
-        self.buttons["Apply"].setEnabled(
+        self.buttons[1].setEnabled(
             all([self.bounds_min_spinbox[i].value() <= self.bounds_max_spinbox[i].value() for i in range(3)])
         )
