@@ -20,45 +20,35 @@ from __future__ import annotations
 from typing import Callable
 import numpy as np
 from numba import jit, prange
+from magneticalc.Backend_Types import get_jit_enabled
 from magneticalc.ConditionalDecorator import ConditionalDecorator
-from magneticalc.Config import get_jit_enabled
 from magneticalc.Constants import Constants
 from magneticalc.Debug import Debug
-from magneticalc.Field_Types import A_FIELD, B_FIELD
+from magneticalc.Field_Types import FIELD_TYPE_A, FIELD_TYPE_B
 from magneticalc.Metric import Metric
+from magneticalc.Validatable import Validatable, require_valid, validator
 
 
-class Parameters:
+class Parameters(Validatable):
     """ Parameters class. """
 
     def __init__(self) -> None:
         """
         Initializes parameters class.
         """
-        Debug(self, ": Init")
+        Validatable.__init__(self)
+        Debug(self, ": Init", init=True)
 
         self._energy: float = 0.0
         self._self_inductance: float = 0.0
         self._magnetic_dipole_moment: float = 0.0
 
-        self._valid = False
-
-    def is_valid(self) -> bool:
+    def set(self, *args, **kwargs):
         """
-        Indicates valid data for display.
-
-        @return: True if data is valid for display, False otherwise
+        Sets the parameters
         """
-        return self._valid
-
-    def invalidate(self) -> None:
-        """
-        Resets data, hiding from display.
-        """
-        Debug(self, ".invalidate()")
-
-        self._valid = False
 
+    @require_valid
     def get_energy(self) -> float:
         """
         Returns calculated energy.
@@ -67,6 +57,7 @@ class Parameters:
         """
         return self._energy
 
+    @require_valid
     def get_self_inductance(self) -> float:
         """
         Returns calculated self-inductance.
@@ -75,6 +66,7 @@ class Parameters:
         """
         return self._self_inductance
 
+    @require_valid
     def get_magnetic_dipole_moment(self) -> float:
         """
         Returns calculated magnetic dipole moment.
@@ -154,6 +146,7 @@ class Parameters:
 
     # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
+    @validator
     def recalculate(
             self,
             wire: Wire,  # type: ignore
@@ -172,19 +165,17 @@ class Parameters:
         """
         Debug(self, ".recalculate()")
 
-        self._valid = False
-
         progress_callback(0)
 
         self._magnetic_dipole_moment = self._get_magnetic_dipole_moment(wire, Metric.LengthScale)
 
         progress_callback(33)
 
-        if field.get_type == A_FIELD:
+        if field.get_type == FIELD_TYPE_A:
 
             pass
 
-        elif field.get_type() == B_FIELD:
+        elif field.get_type() == FIELD_TYPE_B:
 
             dV = (Metric.LengthScale / sampling_volume.get_resolution()) ** 3  # Sampling volume element
             self._energy = self.get_squared_field(sampling_volume, field) * dV / Constants.mu_0
@@ -195,6 +186,4 @@ class Parameters:
 
         progress_callback(100)
 
-        self._valid = True
-
         return True
