@@ -43,7 +43,7 @@ class Perspective_Widget(QGroupBox2):
         @param gui: GUI
         """
         QGroupBox2.__init__(self, "Perspective")
-        Debug(self, ": Init")
+        Debug(self, ": Init", init=True)
         self.gui = gui
 
         planar_perspective_layout = QVBoxLayout()
@@ -60,14 +60,21 @@ class Perspective_Widget(QGroupBox2):
         xyz_hint_layout.addWidget(QLabel2("Z", bold=True, color="#0000cc", align_right=True))
         self.addLayout(xyz_hint_layout)
 
+    def reload(self) -> None:
+        """
+        Reloads the widget.
+        """
+        Debug(self, ".reload()", refresh=True)
+
     def set_perspective(self, preset: Dict) -> None:
         """
         Sets display perspective.
 
-        @param preset: Perspective preset (parameters, see VispyCanvas module)
+        @param preset: Perspective preset (parameters, see VisPyCanvas module)
         """
-        self.gui.config.set_float("azimuth", preset["azimuth"])
-        self.gui.config.set_float("elevation", preset["elevation"])
+        Debug(self, ".set_perspective()")
+        self.gui.project.set_float("azimuth", preset["azimuth"])
+        self.gui.project.set_float("elevation", preset["elevation"])
         self.gui.vispy_canvas.view_main.camera.azimuth = preset["azimuth"]
         self.gui.vispy_canvas.view_main.camera.elevation = preset["elevation"]
         self.gui.redraw()
