@@ -35,16 +35,14 @@ class QButtons(QHBoxLayout):
         """
         QHBoxLayout.__init__(self)
 
-        self.dictionary: Dict[Union[int, str], QPushButton] = {}
+        self.dictionary: Dict[int, QPushButton] = {}
 
         for n, key_val in enumerate(data.items()):
             text, tup = key_val
             icon, callback = tup
 
             button = QPushButton(qta.icon(icon), " " + text)
-            # noinspection PyUnresolvedReferences
-            button.clicked.connect(callback)
+            button.clicked.connect(callback)  # type: ignore
             self.addWidget(button)
 
             self.dictionary[n] = button
-            self.dictionary[text] = button
