@@ -23,8 +23,8 @@ from PyQt5.Qt import QFocusEvent
 from PyQt5.QtCore import Qt, QItemSelectionModel, QItemSelection
 from PyQt5.QtWidgets import QTableWidget, QTableWidgetItem, QHeaderView, QAbstractItemView, QComboBox
 from magneticalc.QPushButton2 import QPushButton2
-from magneticalc.Config import Config
 from magneticalc.Debug import Debug
+from magneticalc.Format import Format
 from magneticalc.Theme import Theme
 
 
@@ -54,7 +54,7 @@ class QTableWidget2(QTableWidget):
         @param minimum_rows: Minimum number of rows (no further rows can be deleted)
         """
         QTableWidget.__init__(self)
-        Debug(self, ": Init")
+        Debug(self, ": Init", init=True)
         self.gui = gui
 
         self._enabled = enabled
@@ -68,14 +68,11 @@ class QTableWidget2(QTableWidget):
         self._options: List = []
 
         if self._cell_edited_callback is not None:
-            # noinspection PyUnresolvedReferences
-            self.itemChanged.connect(self.on_numerical_cell_edited)
-            # noinspection PyUnresolvedReferences
-            self.cellChanged.connect(self.on_cell_changed)
+            self.itemChanged.connect(self.on_numerical_cell_edited)  # type: ignore
+            self.cellChanged.connect(self.on_cell_changed)  # type: ignore
 
         if self._selection_changed_callback is not None:
-            # noinspection PyUnresolvedReferences
-            self.selectionModel().selectionChanged.connect(self.on_selection_changed)
+            self.selectionModel().selectionChanged.connect(self.on_selection_changed)  # type: ignore
 
         self.set_style(border_color="black", border_width=1)
         self.setMinimumHeight(self.MinimumHeight)
@@ -109,14 +106,14 @@ class QTableWidget2(QTableWidget):
 
         @param item: QTableWidgetItem
         """
-        Debug(self, f".on_cell_edited()")
+        Debug(self, ".on_cell_edited()")
 
         try:
             value = float(item.text().replace(",", "."))
         except ValueError:
             value = 0.0
 
-        value = f"{value:.{Config.FloatPrecision}f}"
+        value = Format.float_to_str(value)
         row = item.row()
         column = item.column()
 
@@ -133,7 +130,7 @@ class QTableWidget2(QTableWidget):
         @param row: Row index
         @param column: Column index
         """
-        Debug(self, f".on_combobox_cell_edited()")
+        Debug(self, ".on_combobox_cell_edited()")
 
         if self._cell_edited_callback is not None:
             self._cell_edited_callback(combobox.currentText(), row, column)
@@ -148,10 +145,9 @@ class QTableWidget2(QTableWidget):
         @param _deselected: Currently deselected QItemSelection
         """
         if self.signalsBlocked():
-            Debug(self, f".on_selection_changed(): Blocked")
             return
 
-        Debug(self, f".on_selection_changed()")
+        Debug(self, ".on_selection_changed()")
 
         if self._selection_changed_callback is not None:
             self._selection_changed_callback()
@@ -194,7 +190,7 @@ class QTableWidget2(QTableWidget):
 
         if self._row_deleted_callback is not None:
             if column == self.columnCount() - 1:
-                Debug(self, f".is_cell_widget_selected: Ignoring delete button")
+                Debug(self, ".is_cell_widget_selected: Ignoring delete button")
                 return False
 
         item = self.item(row, column)
@@ -246,10 +242,11 @@ class QTableWidget2(QTableWidget):
         @param focus: Enable to focus the table, disable to explicitly clear the visual selection
         """
         if self.rowCount() == 0:
-            Debug(self, f".select_last_row(): Skipped for empty table")
+            Debug(self, ".select_last_row(): Skipped for empty table")
             return
 
-        Debug(self, f".select_last_row(): Blocking signals")
+        Debug(self, ".select_last_row()")
+
         self.blockSignals(True)
 
         self.select_cell(self.rowCount() - 1, 0)
@@ -260,7 +257,6 @@ class QTableWidget2(QTableWidget):
         else:
             self.clearSelection()
 
-        Debug(self, f".select_last_row(): Unblocking signals")
         self.blockSignals(False)
 
     # ------------------------------------------------------------------------------------------------------------------
@@ -271,7 +267,7 @@ class QTableWidget2(QTableWidget):
 
         @param header: List of row header strings
         """
-        Debug(self, f".set_horizontal_header()")
+        Debug(self, ".set_horizontal_header()")
 
         if self._row_deleted_callback is not None:
             header.append("")
@@ -289,7 +285,7 @@ class QTableWidget2(QTableWidget):
     def set_vertical_prefix(self, prefix: str) -> None:
         """
         Sets the per-row table prefix.
-        Used for mapping cells to configuration; configuration key = prefix + column key + "_" + row index.
+        Used for binding cells to project configuration; key = prefix + column key + "_" + row index.
 
         @param prefix: Prefix
         """
@@ -298,12 +294,12 @@ class QTableWidget2(QTableWidget):
     def set_horizontal_types(self, types: Optional[Dict]) -> None:
         """
         Sets the per-column cell types.
-        Used for mapping cells to configuration.
+        Used for binding cells to project configuration.
 
-        Note: Currently, *only the keys* are used for mapping cells to configuration;
+        Note: Currently, *only the keys* are used for binding cells to project configuration;
         the cell options actually determine the cell type, i.e. numerical or combobox.
 
-        @param types: Ordered dictionary of key-value pairs (column key : column type), or None to disable configuration
+        @param types: Ordered dictionary of key-value pairs (column key : column type), or None to disable binding
         """
         self._types = types
 
@@ -324,7 +320,7 @@ class QTableWidget2(QTableWidget):
 
         @param header: List of column header strings
         """
-        Debug(self, f".set_vertical_header()")
+        Debug(self, ".set_vertical_header()")
 
         self.setRowCount(len(header))
 
@@ -346,7 +342,7 @@ class QTableWidget2(QTableWidget):
         """
         Clears all table rows.
         """
-        Debug(self, f".clear_rows()")
+        Debug(self, ".clear_rows()")
 
         self.setRowCount(0)
 
@@ -358,7 +354,8 @@ class QTableWidget2(QTableWidget):
 
         @param contents: 2D array
         """
-        Debug(self, f".set_contents(): Blocking signals")
+        Debug(self, ".set_contents()")
+
         self.blockSignals(True)
 
         # Iterate over rows
@@ -367,10 +364,6 @@ class QTableWidget2(QTableWidget):
             # Iterate over columns
             for col_index, col_contents in enumerate(row_contents):
 
-                # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-
-                # Determine the configuration and type binding of the cell
-
                 if self._options[col_index] is not None:
                     # Cell has options: Create a combobox
                     combobox = QComboBox()
@@ -378,14 +371,13 @@ class QTableWidget2(QTableWidget):
                     for i, text in enumerate(self._options[col_index]):
                         combobox.addItem(text)
 
-                        # Cell has config:
+                        # Cell is bound to project configuration:
                         if self._types is not None:
                             key = list(self._types.keys())[col_index]
-                            if text == self.gui.config.get_str(self._prefix + key + "_" + str(row_index)):
+                            if text == self.gui.project.get_str(self._prefix + key + "_" + str(row_index)):
                                 combobox.setCurrentIndex(i)
 
-                    # noinspection PyUnresolvedReferences
-                    combobox.currentIndexChanged.connect(
+                    combobox.currentIndexChanged.connect(  # type: ignore
                         partial(self.on_combobox_cell_edited, combobox, row_index, col_index)
                     )
 
@@ -404,10 +396,10 @@ class QTableWidget2(QTableWidget):
                         flags |= Qt.ItemIsEditable
                     item.setFlags(flags)  # type: ignore
 
-                    # Cell has config:
+                    # Cell is bound to project configuration:
                     if self._types is not None:
                         key = list(self._types.keys())[col_index]
-                        item.setText(self.gui.config.get_str(self._prefix + key + "_" + str(row_index)))
+                        item.setText(self.gui.project.get_str(self._prefix + key + "_" + str(row_index)))
 
                     self.setItem(row_index, col_index, item)
 
@@ -418,10 +410,10 @@ class QTableWidget2(QTableWidget):
                 delete_button = QPushButton2("", "fa.minus", css="border: none; background: palette(window);")
 
                 if self.rowCount() > self._minimum_rows:
-                    # noinspection PyUnresolvedReferences
-
                     # Allow this row to be deleted
-                    delete_button.clicked.connect(partial(self.on_row_deleted, row_index))
+                    delete_button.clicked.connect(  # type: ignore
+                        partial(self.on_row_deleted, row_index)
+                    )
                 else:
                     # Don't allow the any more rows to be deleted
                     delete_button.setEnabled(False)
@@ -429,7 +421,6 @@ class QTableWidget2(QTableWidget):
                 # Insert the delete button into the last column
                 self.setCellWidget(row_index, self.columnCount() - 1, delete_button)
 
-        Debug(self, f".set_contents(): Unblocking signals")
         self.blockSignals(False)
 
     # ------------------------------------------------------------------------------------------------------------------
@@ -469,7 +460,7 @@ class QTableWidget2(QTableWidget):
 
         @param _event: QFocusEvent
         """
-        Debug(self, f".focusInEvent()")
+        Debug(self, ".focusInEvent()")
 
         if self.rowCount() > 0:
             self.select_cell()
@@ -479,15 +470,15 @@ class QTableWidget2(QTableWidget):
     def focusOutEvent(self, _event: QFocusEvent) -> None:
         """
         Gets called when the table lost focus, or when a cell item is being edited, or when a cell widget is selected.
-        When not editing, this clears the selection, triggering L{on_selection_changed}
+        When not editing, this clears the selection, triggering L{on_selection_changed}.
 
         @param _event: QFocusEvent
         """
         if self.state() == QAbstractItemView.EditingState:
-            Debug(self, f".focusOutEvent(): Ignored in editing mode")
+            Debug(self, ".focusOutEvent(): Ignored in editing mode")
         elif self.is_cell_widget_selected():
-            Debug(self, f".focusOutEvent(): Ignored for cell widget")
+            Debug(self, ".focusOutEvent(): Ignored for cell widget")
         else:
-            Debug(self, f".focusOutEvent(): Clearing selection")
+            Debug(self, ".focusOutEvent(): Clearing selection")
             self.clearSelection()
             self.set_style(border_color="black", border_width=1)
