@@ -22,27 +22,21 @@ from PyQt5.QtCore import QThread
 from magneticalc.Assert_Dialog import Assert_Dialog
 from magneticalc.Constraint import Constraint
 from magneticalc.Debug import Debug
+from magneticalc.Validatable import Validatable, require_valid, validator
 
 
-class SamplingVolume:
+class SamplingVolume(Validatable):
     """ Sampling volume class. """
 
     # Enable to show additional debug info during constraint calculation
     Debug_Constraints = False
 
-    def __init__(self, resolution: float, label_resolution: float) -> None:
+    def __init__(self) -> None:
         """
         Initializes an empty sampling volume, with zero bounds and no constraints.
-
-        @param resolution: Resolution
-        @param label_resolution: Label resolution
         """
-        Debug(self, ": Init")
-
-        self._resolution = resolution
-        self._label_resolution = label_resolution
-
-        self.constraints = []
+        Validatable.__init__(self)
+        Debug(self, ": Init", init=True)
 
         self._bounds_min = np.zeros(3)
         self._bounds_max = np.zeros(3)
@@ -53,38 +47,38 @@ class SamplingVolume:
         self._labeled_indices: List = []
         self._neighbor_indices: List[np.ndarray] = []
 
-        self._valid = False
+        self._resolution = 0
+        self._label_resolution = 0
+        self._constraints = []
+
+    def set(
+            self,
+            resolution: float,
+            label_resolution: float,
+            constraints: List[Constraint]
+    ) -> None:
+        """
+        Sets the parameters.
+
+        @param resolution: Resolution
+        @param label_resolution: Label resolution
+        @param constraints: List of constraints
+        """
+        self._resolution = resolution
+        self._label_resolution = label_resolution
+        self._constraints = constraints
 
         Assert_Dialog(resolution > 0, "Resolution must be > 0")
         Assert_Dialog(label_resolution > 0, "Label resolution must be > 0")
 
     @property
+    @require_valid
     def dimension(self) -> Tuple[int, int, int]:
         """
-        Gets the sampling volume dimension if it is valid, None otherwise.
-
         @return: Sampling volume dimension if it is valid, None otherwise.
         """
-        Assert_Dialog(self.is_valid(), "Accessing invalidated sampling volume")
-
         return self._dimension
 
-    def is_valid(self) -> bool:
-        """
-        Indicates valid data for display.
-
-        @return: True if data is valid for display, False otherwise
-        """
-        return self._valid
-
-    def invalidate(self) -> None:
-        """
-        Resets data, hiding from display.
-        """
-        Debug(self, ".invalidate()")
-
-        self._valid = False
-
     # ------------------------------------------------------------------------------------------------------------------
 
     def get_resolution(self) -> float:
@@ -111,64 +105,58 @@ class SamplingVolume:
         """
         return self._bounds_max - self._bounds_min
 
+    @require_valid
     def get_points(self) -> np.array:
         """
         Returns this sampling volume's points.
 
         @return: Array of 3D points
         """
-        Assert_Dialog(self.is_valid(), "Accessing invalidated sampling volume")
-
         return self._points
 
+    @require_valid
     def get_points_count(self) -> int:
         """
         Returns this sampling volume's point count.
 
         @return: Point count
         """
-        Assert_Dialog(self.is_valid(), "Accessing invalidated sampling volume")
-
         return len(self._points)
 
+    @require_valid
     def get_permeabilities(self) -> np.ndarray:
         """
         Returns this sampling volume's relative permeabilities µ_r.
 
         @return: Ordered list of sampling volume's relative permeabilities µ_r
         """
-        Assert_Dialog(self.is_valid(), "Accessing invalidated sampling volume")
-
         return self._permeabilities
 
+    @require_valid
     def get_labeled_indices(self) -> List:
         """
         Returns this sampling volume's labeled indices.
 
         @return: Unordered list of pairs [sampling volume point, field index]
         """
-        Assert_Dialog(self.is_valid(), "Accessing invalidated sampling volume")
-
         return self._labeled_indices
 
+    @require_valid
     def get_labels_count(self) -> int:
         """
         Returns this sampling volume's label count.
 
         @return: Label count
         """
-        Assert_Dialog(self.is_valid(), "Accessing invalidated sampling volume")
-
         return len(self._labeled_indices)
 
+    @require_valid
     def get_neighbor_indices(self) -> List[np.ndarray]:
         """
         Returns this sampling volume's neighborhood indices.
 
         @return: Ordered list of sampling volume neighborhood indices (six 3D vectors)
         """
-        Assert_Dialog(self.is_valid(), "Accessing invalidated sampling volume")
-
         return self._neighbor_indices
 
     # ------------------------------------------------------------------------------------------------------------------
@@ -205,18 +193,7 @@ class SamplingVolume:
 
     # ------------------------------------------------------------------------------------------------------------------
 
-    def add_constraint(self, constraint: Constraint) -> None:
-        """
-        Adds some constraint to this volume's point generator.
-
-        @param constraint: Constraint
-        """
-        Debug(self, ".add_constraint()")
-
-        self.constraints.append(constraint)
-
-    # ------------------------------------------------------------------------------------------------------------------
-
+    @validator
     def recalculate(self, progress_callback: Callable) -> bool:
         """
         Recalculates the sampling volume points, permeabilities, labels and neighborhoods according to the constraints.
@@ -226,11 +203,9 @@ class SamplingVolume:
         """
         Debug(self, ".recalculate()")
 
-        self._valid = False
-
         # Group constraints by permeability
         constraints_precedence_dict: Dict = {}
-        for constraint in self.constraints:
+        for constraint in self._constraints:
             if constraint.permeability in constraints_precedence_dict:
                 constraints_precedence_dict[constraint.permeability].append(constraint)
             else:
@@ -428,7 +403,7 @@ class SamplingVolume:
         Debug(
             self,
             ".recalculate(): "
-            f"{len(self.constraints)} constraints left {n} of {len(self._points)} possible points"
+            f"{len(self._constraints)} constraints left {n} of {len(self._points)} possible points"
         )
 
         if len(self._points) == 0:
@@ -441,6 +416,4 @@ class SamplingVolume:
 
         progress_callback(100)
 
-        self._valid = True
-
         return True
