@@ -26,14 +26,13 @@ from magneticalc.QIconLabel import QIconLabel
 from magneticalc.QLabel2 import QLabel2
 from magneticalc.QPushButton2 import QPushButton2
 from magneticalc.QSpinBox2 import QSpinBox2
-from magneticalc.Comparison_Types import comparison_type_from_str
+from magneticalc.Comparison_Types import comparison_name_to_type
 from magneticalc.Constraint import Constraint
 from magneticalc.Constraint_Editor import Constraint_Editor
 from magneticalc.Debug import Debug
 from magneticalc.ModelAccess import ModelAccess
-from magneticalc.Norm_Types import norm_type_from_str
+from magneticalc.Norm_Types import norm_name_to_type
 from magneticalc.OverridePadding_Dialog import OverridePadding_Dialog
-from magneticalc.SamplingVolume import SamplingVolume
 from magneticalc.Theme import Theme
 
 
@@ -78,16 +77,12 @@ class SamplingVolume_Widget(QGroupBox2):
         @param gui: GUI
         """
         QGroupBox2.__init__(self, "Sampling Volume")
-        Debug(self, ": Init")
+        Debug(self, ": Init", init=True)
         self.gui = gui
 
-        # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-
         # Insert constraint editor, but don't fully initialize it yet
         self.constraint_editor = Constraint_Editor(self.gui)
 
-        # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-
         padding_icon_label = QIconLabel("Padding", "mdi.arrow-expand-all")
         padding_override_button = QPushButton2("  Override …  ", "", self.override_padding)
         padding_icon_label.addWidget(padding_override_button)
@@ -149,8 +144,7 @@ class SamplingVolume_Widget(QGroupBox2):
         # Populate resolution combobox
         for i, value in enumerate(self.ResolutionOptionsDict):
             self.resolution_combobox.addItem(str(value))
-        # noinspection PyUnresolvedReferences
-        self.resolution_combobox.currentIndexChanged.connect(
+        self.resolution_combobox.currentIndexChanged.connect(  # type: ignore
             lambda: self.set_sampling_volume(
                 _resolution_exponent_=self.ResolutionOptionsDict.get(
                     self.resolution_combobox.currentText(),
@@ -165,43 +159,78 @@ class SamplingVolume_Widget(QGroupBox2):
         total_points_layout.addWidget(self.total_points_label)
         self.addLayout(total_points_layout)
 
-        # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-
-        self.reinitialize()
-
-    def reinitialize(self) -> None:
+    def reload(self) -> None:
         """
-        Re-initializes the widget and the constraint editor
+        Reloads the widget.
         """
-        Debug(self, ".reinitialize()")
+        Debug(self, ".reload()", refresh=True)
 
         self.blockSignals(True)
 
+        sampling_volume_padding = self.gui.project.get_point("sampling_volume_padding")
         for i in range(3):
-            self.padding_spinbox[i].setValue(self.gui.config.get_point("sampling_volume_padding")[i])
+            self.padding_spinbox[i].setValue(sampling_volume_padding[i])
 
         # Set default resolution if it is not available anymore
-        target = self.gui.config.get_int("sampling_volume_resolution_exponent")
+        target = self.gui.project.get_int("sampling_volume_resolution_exponent")
         if target not in self.ResolutionOptionsDict.values():
             Debug(self, f": WARNING: Invalid: sampling_volume_resolution_exponent = {target}", warning=True)
-            self.gui.config.set_int("sampling_volume_resolution_exponent", 0)
+            self.gui.project.set_int("sampling_volume_resolution_exponent", 0)
 
         # Select the resolution
-        target = self.gui.config.get_int("sampling_volume_resolution_exponent")
+        target = self.gui.project.get_int("sampling_volume_resolution_exponent")
         for i, value in enumerate(self.ResolutionOptionsDict.values()):
             if value == target:
                 self.resolution_combobox.setCurrentIndex(i)
 
         self.blockSignals(False)
 
-        # Re-initialize the constraint editor
-        self.constraint_editor.reinitialize()
+        # Reload the constraint editor
+        self.constraint_editor.reload()
 
-        # Initially load sampling volume from configuration
-        self.set_sampling_volume(recalculate=False, invalidate_self=False)
+        # Initially load sampling volume from project
+        self.set_sampling_volume(recalculate=False, invalidate=False)
 
         self.update()
 
+    def update(self) -> None:
+        """
+        Updates the widget.
+        """
+        Debug(self, ".update()", refresh=True)
+
+        self.update_labels()
+        self.update_controls()
+
+    def update_labels(self) -> None:
+        """
+        Updates the labels.
+        """
+        Debug(self, ".update_labels()", refresh=True)
+
+        if self.gui.model.sampling_volume.valid:
+            self.total_extent_label.setText(
+                " × ".join([f"{extent:.0f}" for extent in self.gui.model.sampling_volume.get_extent()]) + " cm³"
+            )
+            self.total_points_label.setText(
+                str(self.gui.model.sampling_volume.get_points_count())
+            )
+        else:
+            self.total_extent_label.setText("N/A")
+            self.total_points_label.setText("N/A")
+
+        self.total_constraints_label.setText(str(self.constraint_editor.get_count()))
+
+    def update_controls(self) -> None:
+        """
+        Updates the controls.
+        """
+        Debug(self, ".update_controls()", refresh=True)
+
+        self.padding_widget.setEnabled(not self.gui.project.get_bool("sampling_volume_override_padding"))
+
+        self.indicate_valid(self.gui.model.sampling_volume.valid)
+
     # ------------------------------------------------------------------------------------------------------------------
 
     def update_padding(self) -> None:
@@ -221,8 +250,8 @@ class SamplingVolume_Widget(QGroupBox2):
         """
         Debug(self, ".clear_padding()")
 
-        self.gui.config.set_bool("sampling_volume_override_padding", False)
-        self.gui.config.set_points("sampling_volume_bounding_box", [np.zeros(3), np.zeros(3)])
+        self.gui.project.set_bool("sampling_volume_override_padding", False)
+        self.gui.project.set_points("sampling_volume_bounding_box", [np.zeros(3), np.zeros(3)])
 
         self.set_sampling_volume(_padding_=[0, 0, 0])
 
@@ -243,7 +272,7 @@ class SamplingVolume_Widget(QGroupBox2):
         if not dialog.success:
             return
 
-        # This needlessly also clears the override padding and bounding box config, but that's acceptable.
+        # This needlessly also clears the override padding and bounding box settings, but that's acceptable.
         self.clear_padding()
 
         self.set_sampling_volume(
@@ -267,11 +296,9 @@ class SamplingVolume_Widget(QGroupBox2):
 
         self.constraint_editor.show()
 
-        if self.constraint_editor.get_changed():
-
+        if self.constraint_editor.changed:
             self.set_sampling_volume()
-
-            self.constraint_editor.clear_changed()
+            self.constraint_editor.changed = False
 
     # ------------------------------------------------------------------------------------------------------------------
 
@@ -282,8 +309,8 @@ class SamplingVolume_Widget(QGroupBox2):
             _padding_: Optional[List] = None,
             _override_padding_: Optional[bool] = None,
             _bounding_box_: Optional[Tuple[np.ndarray, np.ndarray]] = None,
-            recalculate: bool = True,
-            invalidate_self: bool = True
+            invalidate: bool = True,
+            recalculate: bool = True
     ) -> None:
         """
         Sets the sampling volume. This will overwrite the currently set sampling volume in the model.
@@ -294,8 +321,8 @@ class SamplingVolume_Widget(QGroupBox2):
         @param _padding_: Padding (3D point)
         @param _override_padding_: Enable to override padding instead, setting the bounding box directly
         @param _bounding_box_: Bounding box (used in conjunction with override_padding)
-        @param recalculate: Enable to trigger final re-calculation
-        @param invalidate_self: Enable to invalidate the old sampling volume before setting a new one
+        @param invalidate: Enable to invalidate this model hierarchy level
+        @param recalculate: Enable to trigger final recalculation
         """
         if self.signalsBlocked():
             return
@@ -304,12 +331,12 @@ class SamplingVolume_Widget(QGroupBox2):
 
         with ModelAccess(self.gui, recalculate):
 
-            resolution_exponent = self.gui.config.set_get_int(
+            resolution_exponent = self.gui.project.set_get_int(
                 "sampling_volume_resolution_exponent",
                 _resolution_exponent_
             )
 
-            label_resolution_exponent = self.gui.config.set_get_int(
+            label_resolution_exponent = self.gui.project.set_get_int(
                 "sampling_volume_label_resolution_exponent",
                 _label_resolution_exponent_
             )
@@ -317,32 +344,35 @@ class SamplingVolume_Widget(QGroupBox2):
             resolution = np.power(2.0, resolution_exponent)
             label_resolution = np.power(2.0, label_resolution_exponent)
 
-            self.gui.model.set_sampling_volume(
-                SamplingVolume(resolution=resolution, label_resolution=label_resolution),
-                invalidate_self=invalidate_self
-            )
-
-            self.readjust(_padding_=_padding_, _override_padding_=_override_padding_, _bounding_box_=_bounding_box_)
-
-            # Load sampling volume constraints from configuration
+            # Convert every line in the constraint editor to a Constraint() object
+            constraints = []
             for constraint in self.constraint_editor.get_constraints():
-                Debug(self, f".set_sampling_volume(): Loading Constraint: {constraint}")
-                self.gui.model.sampling_volume.add_constraint(
+                Debug(self, f".set_sampling_volume(): Constraint: {constraint}")
+                constraints.append(
                     Constraint(
-                        norm_type_from_str(constraint["norm"]),
-                        comparison_type_from_str(constraint["comparison"]),
+                        norm_name_to_type(constraint["norm"]),
+                        comparison_name_to_type(constraint["comparison"]),
                         constraint["min"],
                         constraint["max"],
                         constraint["permeability"]
                     )
                 )
 
+            self.gui.model.set_sampling_volume(
+                invalidate=invalidate,
+                resolution=resolution,
+                label_resolution=label_resolution,
+                constraints=constraints
+            )
+
+            self.readjust(_padding_=_padding_, _override_padding_=_override_padding_, _bounding_box_=_bounding_box_)
+
             if recalculate:
                 # The display widget depends on the sampling volume
                 self.gui.sidebar_right.display_widget.update()
 
                 # This forces updating the number of constraints if the sampling volume is already invalidated
-                if not self.gui.model.sampling_volume.is_valid():
+                if not self.gui.model.sampling_volume.valid:
                     self.update()
 
     # ------------------------------------------------------------------------------------------------------------------
@@ -367,8 +397,8 @@ class SamplingVolume_Widget(QGroupBox2):
         """
         Debug(self, ".readjust()")
 
-        override_padding = self.gui.config.set_get_bool("sampling_volume_override_padding", _override_padding_)
-        bounding_box = self.gui.config.set_get_points("sampling_volume_bounding_box", _bounding_box_)
+        override_padding = self.gui.project.set_get_bool("sampling_volume_override_padding", _override_padding_)
+        bounding_box = self.gui.project.set_get_points("sampling_volume_bounding_box", _bounding_box_)
 
         if override_padding:
 
@@ -380,7 +410,7 @@ class SamplingVolume_Widget(QGroupBox2):
 
             bounds_min, bounds_max = self.gui.model.sampling_volume.get_bounds()
 
-            padding = self.gui.config.set_get_point("sampling_volume_padding", _padding_)
+            padding = self.gui.project.set_get_point("sampling_volume_padding", _padding_)
 
             # Adjust padding values if they are now outside the new minimum padding bounds
             padding_adjusted = False
@@ -390,7 +420,7 @@ class SamplingVolume_Widget(QGroupBox2):
                     padding_adjusted = True
             if padding_adjusted:
                 Debug(self, ".readjust(): Adjusted padding values to new minimum padding bounds")
-                self.gui.config.set_point("sampling_volume_padding", padding)
+                self.gui.project.set_point("sampling_volume_padding", padding)
 
             self.blockSignals(True)
             for i in range(3):
@@ -398,43 +428,3 @@ class SamplingVolume_Widget(QGroupBox2):
             self.blockSignals(False)
 
             self.gui.model.sampling_volume.set_padding_nearest(padding)
-
-    # ------------------------------------------------------------------------------------------------------------------
-
-    def update(self) -> None:
-        """
-        Updates the widget.
-        """
-        Debug(self, ".update()")
-
-        self.update_labels()
-        self.update_controls()
-
-    def update_labels(self) -> None:
-        """
-        Updates the labels.
-        """
-        Debug(self, ".update_labels()")
-
-        if self.gui.model.sampling_volume.is_valid():
-            self.total_extent_label.setText(
-                " × ".join([f"{extent:.0f}" for extent in self.gui.model.sampling_volume.get_extent()]) + " cm³"
-            )
-            self.total_points_label.setText(
-                str(self.gui.model.sampling_volume.get_points_count())
-            )
-        else:
-            self.total_extent_label.setText("N/A")
-            self.total_points_label.setText("N/A")
-
-        self.total_constraints_label.setText(str(self.constraint_editor.get_count()))
-
-    def update_controls(self) -> None:
-        """
-        Updates the controls.
-        """
-        Debug(self, ".update_controls()")
-
-        self.padding_widget.setEnabled(not self.gui.config.get_bool("sampling_volume_override_padding"))
-
-        self.indicate_valid(self.gui.model.sampling_volume is not None and self.gui.model.sampling_volume.is_valid())
