@@ -22,7 +22,7 @@ from multiprocessing import cpu_count
 from sty import fg
 from PyQt5.QtWidgets import QWidget, QHBoxLayout, QCheckBox, QComboBox, QProgressBar, QLabel
 from magneticalc.QPushButton2 import QPushButton2
-from magneticalc.Config import get_jit_enabled
+from magneticalc.Backend_Types import get_jit_enabled
 from magneticalc.Debug import Debug
 from magneticalc.Theme import Theme
 
@@ -42,9 +42,10 @@ class Statusbar:
 
         @param gui: GUI
         """
-        Debug(self, ": Init")
+        Debug(self, ": Init", init=True)
         self.gui = gui
 
+        self._valid = True  # Note: Needs to be true initially, otherwise the first L{invalidate()} would have no effect
         self._canceled = False
 
         # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -57,8 +58,7 @@ class Statusbar:
 
         # Auto-calculation checkbox
         self._auto_calculation_checkbox = QCheckBox("Auto-Calculation")
-        # noinspection PyUnresolvedReferences
-        self._auto_calculation_checkbox.toggled.connect(self._auto_calculation_changed)
+        self._auto_calculation_checkbox.toggled.connect(self._auto_calculation_changed)  # type: ignore
 
         # Number-of-cores combobox
         self._cores_combobox = QComboBox()
@@ -76,9 +76,8 @@ class Statusbar:
                 else:
                     self._cores_combobox.addItem(f"{i} Core" + ("s" if i > 1 else ""))
 
-            # noinspection PyUnresolvedReferences
-            self._cores_combobox.currentIndexChanged.connect(
-                lambda: self.gui.config.set_int("num_cores", self._cores_combobox.currentIndex())
+            self._cores_combobox.currentIndexChanged.connect(  # type: ignore
+                lambda: self.gui.project.set_int("num_cores", self._cores_combobox.currentIndex())
             )
 
         else:
@@ -117,27 +116,28 @@ class Statusbar:
 
         gui.statusBar().setSizeGripEnabled(False)
 
-        self.reinitialize()
-
-    def reinitialize(self) -> None:
+    def reload(self) -> None:
         """
-        Re-initializes the statusbar.
+        Reloads the statusbar.
         """
-        Debug(self, ".reinitialize()")
+        Debug(self, ".reload()", refresh=True)
 
         self.gui.blockSignals(True)
 
-        self._auto_calculation_checkbox.setChecked(self.gui.config.get_bool("auto_calculation"))
-        if 0 > self.gui.config.get_int("num_cores") > cpu_count():
-            self.gui.config.set_int("num_cores", 0)
+        self._auto_calculation_checkbox.setChecked(self.gui.project.get_bool("auto_calculation"))
+        if 0 > self.gui.project.get_int("num_cores") > cpu_count():
+            self.gui.project.set_int("num_cores", 0)
 
         if get_jit_enabled():
+            num_cores = self.gui.project.get_int("num_cores")
             for i in range(0, cpu_count() + 1):
-                if i == self.gui.config.get_int("num_cores"):
+                if i == num_cores:
                     self._cores_combobox.setCurrentIndex(i)
 
         self.gui.blockSignals(False)
 
+        self.invalidate()
+
     # ------------------------------------------------------------------------------------------------------------------
 
     def _auto_calculation_changed(self) -> None:
@@ -149,9 +149,9 @@ class Statusbar:
 
         self._canceled = False
 
-        self.gui.config.set_bool("auto_calculation", self._auto_calculation_checkbox.isChecked())
+        self.gui.project.set_bool("auto_calculation", self._auto_calculation_checkbox.isChecked())
         if self._auto_calculation_checkbox.isChecked():
-            if not self.gui.model.is_valid():
+            if not self.gui.model.valid:
                 self.gui.recalculate()
 
     # ------------------------------------------------------------------------------------------------------------------
@@ -160,8 +160,9 @@ class Statusbar:
         """
         "Arms" the statusbar before calculation.
         """
-        Debug(self, f".arm()")
+        Debug(self, ".arm()")
 
+        self._valid = False
         self._canceled = False
 
         self._start_button.setEnabled(False)
@@ -182,6 +183,7 @@ class Statusbar:
         """
         Debug(self, f".disarm(success={success})")
 
+        self._valid = success
         self._canceled = not success
 
         self._start_button.setEnabled(True)
@@ -201,14 +203,20 @@ class Statusbar:
         """
         "Invalidates" the statusbar.
         """
-        Debug(self, ".invalidate()")
 
-        # Skip if the calculation was canceled
+        # Don't invalidate more than once
+        if not self._valid:
+            return
+
+        # Don't invalidate after cancellation
         if self._canceled:
-            Debug(self, ".invalidate(): Skipped due to previously canceled calculation")
             self._canceled = False
             return
 
+        Debug(self, ".invalidate()")
+
+        self._valid = False
+
         self._start_button.setEnabled(True)
         self._cancel_button.setEnabled(False)
         self._auto_calculation_checkbox.setEnabled(True)
@@ -235,7 +243,9 @@ class Statusbar:
         """
         Debug(self, ".cancel()")
 
-        self.gui.interrupt_calculation()
+        if self.gui.calculation_thread is not None:
+            self.gui.interrupt_calculation()
+
         self.disarm(False)
 
     def set_text(self, text: str) -> None:
