@@ -21,12 +21,31 @@ import numpy as np
 from PyQt5.QtCore import QThread
 from magneticalc.Assert_Dialog import Assert_Dialog
 from magneticalc.Debug import Debug
+from magneticalc.Validatable import Validatable, require_valid, validator
 
 
-class Wire:
+class Wire(Validatable):
     """ Wire class. """
 
-    def __init__(
+    def __init__(self) -> None:
+        """
+        Initializes an empty wire.
+        A 3D piecewise linear curve with some DC current associated with it.
+        """
+        Validatable.__init__(self)
+        Debug(self, ": Init", init=True)
+
+        self._points_base = np.array([])
+
+        self._slicer_limit = 0
+        self._dc = 0
+
+        self._points_sliced = np.array([])
+        self._length = 0
+
+        self._points_transformed = np.array([])
+
+    def set(
             self,
             points: np.ndarray,
             stretch: np.ndarray,
@@ -36,7 +55,7 @@ class Wire:
             dc: float
     ) -> None:
         """
-        A 3D piecewise linear curve with some DC current associated with it.
+        Sets the parameters.
 
         @param points: Ordered list of 3D coordinates (see presets)
         @param stretch: XYZ stretch transform factors (3D point)
@@ -45,42 +64,14 @@ class Wire:
         @param slicer_limit: Slicer limit
         @param dc: Wire current (A)
         """
-        Debug(self, ": Init")
-
         self._points_base = np.array(points)
 
-        # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-        # Note: This is my playground for creating new wire presets!
-        override_base = False
-        if override_base:
-            self._points_base = np.array(
-                [
-                    [
-                        i / 128 - 1.125,
-                        -np.cos(+2 * np.pi * i / 16) / 2,
-                        +np.sin(+2 * np.pi * i / 16) / 2
-                    ]
-                    for i in range(128)
-                ] +
-                [
-                    [
-                        i / 128 + 0.125,
-                        -np.cos(-2 * np.pi * (i + 1) / 16) / 2,
-                        +np.sin(-2 * np.pi * (i + 1) / 16) / 2
-                    ]
-                    for i in range(128)
-                ]
-            )
-        # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-
         self._slicer_limit = slicer_limit
         self._dc = dc
 
         self._points_sliced = np.array([])
         self._length = 0
 
-        self._valid = False
-
         self._points_transformed = self._points_base.copy()
 
         self._set_stretch(stretch)
@@ -88,24 +79,14 @@ class Wire:
 
         Assert_Dialog(len(self._points_base) >= 2, "Number of points must be >= 2")
 
-    def is_valid(self) -> bool:
-        """
-        Indicates valid data for display.
-
-        @return: True if data is valid for display, False otherwise
-        """
-        return self._valid
-
-    def invalidate(self) -> None:
-        """
-        Resets data, hiding from display.
-        """
-        Debug(self, ".invalidate()")
-
-        self._valid = False
-
     # ------------------------------------------------------------------------------------------------------------------
 
+    def get_dc(self) -> float:
+        """
+        @return: Wire current (A)
+        """
+        return self._dc
+
     def get_bounds(self) -> Tuple[List, List]:
         """
         Returns this curve's bounding box.
@@ -133,24 +114,22 @@ class Wire:
         """
         return self._points_transformed
 
+    @require_valid
     def get_points_sliced(self) -> np.ndarray:
         """
         Returns this wire's points after slicing.
 
         @return: Ordered list of 3D points
         """
-        Assert_Dialog(self.is_valid(), "Accessing invalidated wire")
-
         return self._points_sliced
 
+    @require_valid
     def get_elements(self) -> np.ndarray:
         """
         Returns this curve's elements, i.e. an ordered list of segment center points and directions.
 
         @return: [[element_center, element_direction], …]
         """
-        Assert_Dialog(self.is_valid(), "Accessing invalidated wire")
-
         result = []
 
         for i in range(len(self._points_sliced) - 1):
@@ -160,22 +139,11 @@ class Wire:
 
         return np.array(result)
 
-    def get_dc(self) -> float:
-        """
-        Returns wire current (A).
-
-        @return: Wire current (A)
-        """
-        return self._dc
-
+    @require_valid
     def get_length(self) -> float:
         """
-        Returns this curve's length.
-
-        @return: Length
+        @return: Wire length
         """
-        Assert_Dialog(self.is_valid(), "Accessing invalidated wire")
-
         return self._length
 
     # ------------------------------------------------------------------------------------------------------------------
@@ -233,6 +201,7 @@ class Wire:
 
     # ------------------------------------------------------------------------------------------------------------------
 
+    @validator
     def recalculate(self, progress_callback: Callable) -> bool:
         """
         Slices wire segments into smaller ones until segment lengths equal or undershoot slicer limit.
@@ -242,8 +211,6 @@ class Wire:
         """
         Debug(self, ".recalculate()")
 
-        self._valid = False
-
         points_sliced = []
         length = 0
 
@@ -275,6 +242,4 @@ class Wire:
 
         progress_callback(100)
 
-        self._valid = True
-
         return True
