@@ -32,7 +32,19 @@ from magneticalc.QTableWidget2 import QTableWidget2
 from magneticalc.Debug import Debug
 from magneticalc.ModelAccess import ModelAccess
 from magneticalc.Theme import Theme
-from magneticalc.Wire import Wire
+
+
+""" Default stretch setting. """
+DefaultStretch = [1.0, 1.0, 1.0]
+
+
+""" Default rotational symmetry setting. """
+DefaultRotationalSymmetry = {
+    "count" : 1,
+    "radius": 0,
+    "axis"  : 2,
+    "offset": 0
+}
 
 
 class Wire_Widget(QGroupBox2):
@@ -75,7 +87,7 @@ class Wire_Widget(QGroupBox2):
         @param gui: GUI
         """
         QGroupBox2.__init__(self, "Wire")
-        Debug(self, ": Init")
+        Debug(self, ": Init", init=True)
         self.gui = gui
 
         # --------------------------------------------------------------------------------------------------------------
@@ -182,8 +194,9 @@ class Wire_Widget(QGroupBox2):
         rotational_symmetry_layout_left.addWidget(QLabel2("Axis:"))
         rotational_symmetry_layout_middle.addWidget(self.rotational_symmetry_axis_combobox)
         rotational_symmetry_layout_right.addWidget(QLabel2("", width=self.UnitsLabelWidth))
-        # noinspection PyUnresolvedReferences
-        self.rotational_symmetry_axis_combobox.currentIndexChanged.connect(self.set_rotational_symmetry)
+        self.rotational_symmetry_axis_combobox.currentIndexChanged.connect(  # type: ignore
+            self.set_rotational_symmetry
+        )
 
         # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
@@ -208,8 +221,7 @@ class Wire_Widget(QGroupBox2):
         self.addWidget(QHLine())
 
         self.close_loop_checkbox = QCheckBox(" Close Loop")
-        # noinspection PyUnresolvedReferences
-        self.close_loop_checkbox.toggled.connect(
+        self.close_loop_checkbox.toggled.connect(  # type: ignore
             lambda: self.set_wire(_close_loop_=self.close_loop_checkbox.isChecked())
         )
         self.addWidget(self.close_loop_checkbox)
@@ -274,42 +286,68 @@ class Wire_Widget(QGroupBox2):
         dc_layout.addWidget(QLabel2("A", align_right=True, width=self.UnitsLabelWidth))
         self.addLayout(dc_layout)
 
-        # --------------------------------------------------------------------------------------------------------------
-
-        self.reinitialize()
-
-    def reinitialize(self) -> None:
+    def reload(self) -> None:
         """
-        Re-initializes the widget.
+        Reloads the widget.
         """
-        Debug(self, ".reinitialize()")
+        Debug(self, ".reload()", refresh=True)
 
         self.blockSignals(True)
 
+        wire_stretch = self.gui.project.get_point("wire_stretch")
         for i in range(3):
-            self.stretch_spinbox[i].setValue(self.gui.config.get_point("wire_stretch")[i])
+            self.stretch_spinbox[i].setValue(wire_stretch[i])
 
-        self.rotational_symmetry_count_spinbox.setValue(self.gui.config.get_float("rotational_symmetry_count"))
-        self.rotational_symmetry_radius_spinbox.setValue(self.gui.config.get_float("rotational_symmetry_radius"))
+        self.rotational_symmetry_count_spinbox.setValue(self.gui.project.get_float("rotational_symmetry_count"))
+        self.rotational_symmetry_radius_spinbox.setValue(self.gui.project.get_float("rotational_symmetry_radius"))
 
+        rotational_symmetry_axis = self.gui.project.get_int("rotational_symmetry_axis")
         for i, axis in enumerate(["X", "Y", "Z"]):
-            if i == self.gui.config.get_int("rotational_symmetry_axis"):
+            if i == rotational_symmetry_axis:
                 self.rotational_symmetry_axis_combobox.setCurrentIndex(i)
 
-        self.rotational_symmetry_offset_spinbox.setValue(self.gui.config.get_float("rotational_symmetry_offset"))
+        self.rotational_symmetry_offset_spinbox.setValue(self.gui.project.get_float("rotational_symmetry_offset"))
 
-        self.close_loop_checkbox.setChecked(self.gui.config.get_bool("wire_close_loop"))
+        self.close_loop_checkbox.setChecked(self.gui.project.get_bool("wire_close_loop"))
 
-        self.slicer_limit_spinbox.setValue(self.gui.config.get_float("wire_slicer_limit"))
-        self.dc_spinbox.setValue(self.gui.config.get_float("wire_dc"))
+        self.slicer_limit_spinbox.setValue(self.gui.project.get_float("wire_slicer_limit"))
+        self.dc_spinbox.setValue(self.gui.project.get_float("wire_dc"))
 
         self.blockSignals(False)
 
-        # Initially load wire from configuration
-        self.set_wire(recalculate=False, readjust_sampling_volume=False, invalidate_self=False)
+        # Initially load wire from project
+        self.set_wire(recalculate=False, readjust_sampling_volume=False, invalidate=False)
 
         self.update()
 
+    def update(self) -> None:
+        """
+        Updates the widget.
+        """
+        Debug(self, ".update()", refresh=True)
+
+        self.update_labels()
+        self.update_controls()
+
+    def update_labels(self) -> None:
+        """
+        Updates the labels.
+        """
+        Debug(self, ".update_labels()", refresh=True)
+
+        if self.gui.model.wire.valid:
+            self.sliced_total_label.setText(str(len(self.gui.model.wire.get_points_sliced())))
+        else:
+            self.sliced_total_label.setText("N/A")
+
+    def update_controls(self) -> None:
+        """
+        Updates the controls.
+        """
+        Debug(self, ".update_controls()", refresh=True)
+
+        self.indicate_valid(self.gui.model.wire.valid)
+
     # ------------------------------------------------------------------------------------------------------------------
 
     def update_table(self) -> None:
@@ -456,9 +494,9 @@ class Wire_Widget(QGroupBox2):
             _close_loop_: Optional[bool] = None,
             _slicer_limit_: Optional[float] = None,
             _dc_: Optional[float] = None,
+            invalidate: bool = True,
             recalculate: bool = True,
-            readjust_sampling_volume: bool = True,
-            invalidate_self: bool = True
+            readjust_sampling_volume: bool = True
     ) -> None:
         """
         Sets the wire. This will overwrite the currently set wire in the model.
@@ -472,9 +510,9 @@ class Wire_Widget(QGroupBox2):
         @param _close_loop_: Enable to transform the wire into a closed loop (append first point)
         @param _slicer_limit_: Slicer limit
         @param _dc_: DC value
-        @param recalculate: Enable to trigger final re-calculation
+        @param invalidate: Enable to invalidate this model hierarchy level
+        @param recalculate: Enable to trigger final recalculation
         @param readjust_sampling_volume: Enable to readjust sampling volume
-        @param invalidate_self: Enable to invalidate the old wire before setting a new one
         """
         if self.signalsBlocked():
             return
@@ -486,13 +524,13 @@ class Wire_Widget(QGroupBox2):
             should_update_stretch_controls = _stretch_ is not None
             should_update_rotational_symmetry_controls = _rotational_symmetry_ is not None
 
-            points = self.gui.config.set_get_points("wire_points_base", _points_)
-            stretch = self.gui.config.set_get_point("wire_stretch", _stretch_)
-            close_loop = self.gui.config.set_get_bool("wire_close_loop", _close_loop_)
-            slicer_limit = self.gui.config.set_get_float("wire_slicer_limit", _slicer_limit_)
-            dc = self.gui.config.set_get_float("wire_dc", _dc_)
+            points = self.gui.project.set_get_points("wire_points_base", _points_)
+            stretch = self.gui.project.set_get_point("wire_stretch", _stretch_)
+            close_loop = self.gui.project.set_get_bool("wire_close_loop", _close_loop_)
+            slicer_limit = self.gui.project.set_get_float("wire_slicer_limit", _slicer_limit_)
+            dc = self.gui.project.set_get_float("wire_dc", _dc_)
 
-            rotational_symmetry = self.gui.config.set_get_dict(
+            rotational_symmetry = self.gui.project.set_get_dict(
                 prefix="rotational_symmetry_",
                 suffix="",
                 types={"count": "int", "radius": "float", "axis": "int", "offset": "float"},
@@ -500,15 +538,13 @@ class Wire_Widget(QGroupBox2):
             )
 
             self.gui.model.set_wire(
-                Wire(
-                    points=points,
-                    stretch=stretch,
-                    rotational_symmetry=rotational_symmetry,
-                    close_loop=close_loop,
-                    slicer_limit=slicer_limit,
-                    dc=dc
-                ),
-                invalidate_self=invalidate_self
+                invalidate=invalidate,
+                points=points,
+                stretch=stretch,
+                rotational_symmetry=rotational_symmetry,
+                close_loop=close_loop,
+                slicer_limit=slicer_limit,
+                dc=dc
             )
 
             self.update_table()
@@ -523,33 +559,3 @@ class Wire_Widget(QGroupBox2):
 
             if readjust_sampling_volume:
                 self.gui.sidebar_left.sampling_volume_widget.readjust()
-
-    # ------------------------------------------------------------------------------------------------------------------
-
-    def update(self) -> None:
-        """
-        Updates the widget.
-        """
-        Debug(self, ".update()")
-
-        self.update_labels()
-        self.update_controls()
-
-    def update_labels(self) -> None:
-        """
-        Updates the labels.
-        """
-        Debug(self, ".update_labels()")
-
-        if self.gui.model.wire.is_valid():
-            self.sliced_total_label.setText(str(len(self.gui.model.wire.get_points_sliced())))
-        else:
-            self.sliced_total_label.setText("N/A")
-
-    def update_controls(self) -> None:
-        """
-        Updates the controls.
-        """
-        Debug(self, ".update_controls()")
-
-        self.indicate_valid(self.gui.model.wire is not None and self.gui.model.wire.is_valid())
